
#Region "License Agreement"

'* DigitArq
'* Copyright 2007-2015
'* Arquivo Distrital do Porto. Todos os direitos
'* reservados.
'*  
'* Este software foi desenvolvido pelo Arquivo
'* Distrital do Porto (http://www.adporto.org)
'* sob a administrao da Direco-Geral de
'* Arquivos (http://www.dgarq.gov.pt) e debaixo da
'* coordenao tcnica da Universidade do Minho
'* (http://www.uminho.pt).
'* O desenvolvimento deste produto foi parcialmente
'* financiado pelo Programa Operacional para a
'* Cultura (POC) promovido pelo Governo Portugus.
'* ---------------------------------------------------
'*
'* A redistribuio e utilizao deste produto sob a
'* forma de cdigo-fonte ou programa compilado, com ou
'* sem modificaes,  permitida desde que o seguinte
'* conjunto de condies seja cumprido:
'* 
'*     * Redistribuies do cdigo-fonte deste produto
'*	  devero ser acompanhadas do texto que compe
'*	  esta licena, incluindo o texto inicial de
'*	  atribuio de autoria, esta lista de condies
'*	  e o seguinte termo de responsabilidade.
'*     * Redistribuies deste produto sob a forma de
'*	  programa compilado devero ser acompanhadas do
'*	  texto que compe esta licena, incluindo o
'*	  texto inicial de atribuio de autoria, esta
'*	  lista de condies e o seguinte termo de
'*	  responsabilidade na documentao e outros
'*	  materiais que acompanhem a distribuio.
'*     * Os nomes da Direco-Geral de Arquivos, do
'*	  Arquivo Distrital do Porto, da Universidade do
'*	  Minho e das pessoas individuais que colaboraram
'*	  no desenvolvimento deste produto no devero
'*	  ser utilizados na promoo de produtos
'*	  derivados deste sem que seja obtido
'*	  consentimento prvio, por escrito, por parte
'*	  dos visados.
'*     * A utilizao da designao DigitArq, seus
'*	  logtipos e nomesinstitucionais associados 
'*	  apenas permitida em distribuies que sejam
'*	  cpias exactas da verso oficial deste produto
'*	  provada e/ou distribuida pela Direco-Geral de
'*	  Arquivos.
'*     * O desenvolvimento de obras derivadas deste
'*	  produto  permitido desde que a designao
'*	  DigitArq, seus logtipos e parceiros
'*	  institucionais no sejam utilizados em todo e
'*	  qualquer tipo de distribuio e/ou promoo da
'*	  obra derivada.
'* 
'* ESTE SOFTWARE  DISTRIBUIDO PELA DIRECO-GERAL DE
'* ARQUIVOS SEM QUALQUER PRESUNO DE QUALIDADE OU
'* GARANTIA ASSOCIADA, INCLUINDO, MAS NO LIMITADO A,
'* GARANTIAS ASSOCIADAS A COMRCIO DE PRODUTOS OU
'* AFIRMAO DE ADEQUABILIDADE A DETERMINADO FIM OU
'* OBJECTIVO. EM QUALQUER CIRCUNSTNCIA DEVER A
'* DIRECO-GERAL DE ARQUIVOS SER CONSIDERADA
'* RESPONSVEL POR QUALQUER DANO QUE POSSA SER PROVOCADO
'* DA UTILIZAO DIRECTA, INDIRECTA, ACIDENTAL, ESPECIAL 
'* OU DEMONSTRATIVA DESTE PRODUTO (INCLUINDO, MAS NO
'* LIMITADO A, PERDAS DE DADOS, LUCROS, FALNCIA,
'* INDEVIDA PRESTAO DE SERVIOS, NEGLIGNCIA, ETC).
'* ---------------------------------------------------
'*
'* Para mais informao sobre este produto ou a sua
'* licena,  favor consultar o endereo electrnico
'* http://www.digitarq.pt

#End Region

Imports System.Threading
Imports System.IO
Imports System.Data.SqlClient
Imports System.Configuration
Imports System.Text.RegularExpressions
Imports System.Xml

Imports Digitarq.Utility
Imports Digitarq.Data.Entities.MDA
Imports Digitarq.Data.Entities.MGOD
'Imports Digitarq.Data.Entities

Imports Digitarq.Definitions.MGOD
Imports Digitarq.Definitions.EAD
Imports Digitarq.Core.Logging
Imports Digitarq.UserControls.ImageButton
Imports Digitarq.UserControls

Public Class Importer

    Private myServerAddress As String
    Private myDatabase As String
    Private myUsername As String
    Private myPassword As String
    Private myFilename As String
    Private myProgressDialog As ProgressDialog.ProgressDialog
    Private myProcessInfoName As String = "admin"
    Private mySourceFonds As LazyNode
    Private myDestinationFonds As LazyNode
    Private myImportType As ImportType
    Private myErrorCount As Integer
    Private DIF_IMPORTER_LOG_PATH As String = Environment.GetFolderPath(Environment.SpecialFolder.Personal) & "\dif_importer.log"


    Public Enum ImportType
        ExternalEAD
        TabSeparatedFile
        ExistingFonds
        CALMRecords
        DigitArqIntoroperabilityFormat
    End Enum


    Public ReadOnly Property ErrorCount()
        Get
            Return myErrorCount
        End Get
    End Property




    Public Sub New(ByVal ServerAddress As String, ByVal Database As String, _
                   ByVal Username As String, ByVal Password As String, _
                   ByVal Filename As String, ByVal ProcessInfoName As String)
        myServerAddress = ServerAddress
        myDatabase = Database
        myUsername = Username
        myPassword = Password
        myFilename = Filename
        myProcessInfoName = ProcessInfoName
        myImportType = ImportType.ExternalEAD
    End Sub


    Public Sub New(ByVal ServerAddress As String, ByVal Database As String, _
                   ByVal Username As String, ByVal Password As String, _
                   ByVal Filename As String, ByVal ProcessInfoName As String, _
                   ByVal ImportType As ImportType)
        myServerAddress = ServerAddress
        myDatabase = Database
        myUsername = Username
        myPassword = Password
        myFilename = Filename
        myProcessInfoName = ProcessInfoName
        myImportType = ImportType
    End Sub





    Public Sub New(ByVal DestinationFonds As LazyNode, ByVal SourceFonds As LazyNode)
        myDestinationFonds = DestinationFonds
        mySourceFonds = SourceFonds
        myImportType = ImportType.ExistingFonds

    End Sub


    Public Sub New(ByVal DestinationFonds As LazyNode, ByVal Filename As String, ByVal ProcessInfoName As String)
        myDestinationFonds = DestinationFonds
        myFilename = Filename
        myProcessInfoName = ProcessInfoName
        myImportType = ImportType.TabSeparatedFile
    End Sub


    Public Sub Import()
        Select Case myImportType
            Case ImportType.ExistingFonds : ImportExistingFonds()
            Case ImportType.ExternalEAD : ImportExternalEAD()
            Case ImportType.TabSeparatedFile : ImportTabSeparatedFile()
            Case ImportType.CALMRecords : ImportCALMRecords()
            Case ImportType.DigitArqIntoroperabilityFormat : ImportDIF()
        End Select
    End Sub



    Public Sub Import(ByVal ImportType As ImportType)
        Select Case ImportType
            Case ImportType.ExistingFonds : ImportExistingFonds()
            Case ImportType.ExternalEAD : ImportExternalEAD()
            Case ImportType.TabSeparatedFile : ImportTabSeparatedFile()
            Case ImportType.CALMRecords : ImportCALMRecords()
            Case ImportType.DigitArqIntoroperabilityFormat : ImportDIF()
        End Select
    End Sub


#Region "Import External EAD"





    Private Sub ImportExternalEAD()
        Cursor.Current = Cursors.WaitCursor
        Dim startTime As Date = Date.Now
        myProgressDialog = New ProgressDialog.ProgressDialog(True)
        myProgressDialog.MinimumValue = 0
        myProgressDialog.MaximumValue = 20 ' mySourceFonds.DescendantsCount()
        myProgressDialog.Text = String.Format(InfoMessage("Importer.Title"), Path.GetFileName(myFilename))
        '"A importar " & Path.GetFileName(myFilename) & "..."
        myProgressDialog.Show()

        Dim t As Thread = New Thread(New ThreadStart(AddressOf ImportFonds))
        't.IsBackground = True
        t.Start()

        While t.IsAlive
            System.Windows.Forms.Application.DoEvents()
        End While

        'ImportFonds()

        myProgressDialog.Hide()
        myProgressDialog.Close()
        myProgressDialog.Dispose()

        Dim finishTime As Date = Date.Now
        Dim timeSpan As timeSpan = finishTime.Subtract(startTime)
        Debug.WriteLine(Path.GetFileName(myFilename) & ": " & timeSpan.Minutes.ToString.PadLeft(2, "0") & ":" & timeSpan.Seconds.ToString.PadLeft(2, "0") & " minutos")

        Cursor.Current = Cursors.Default
    End Sub

    Private Sub MergeTrees(ByVal DstNode As LazyNode, ByVal SrcNode As LazyNode)
        Dim Index As Integer
        Dim ChildSrcNode As LazyNode
        Dim Children As LazyNodeCollection


        Children = DstNode.Children
        For Each ChildSrcNode In SrcNode.Children
            Index = Children.IndexOf(ChildSrcNode)
            myProgressDialog.Message = String.Format(InfoMessage("Importer.Progression"), """" & ChildSrcNode.UnitId & " - " & ChildSrcNode.UnitTitle & """")
            myProgressDialog.Increment()

            If myProgressDialog.Cancel Then Exit Sub

            If Index < 0 OrElse ChildSrcNode.isLeaf Then ' node doesn't exist. Create new one
                Dim NewDstNode As LazyNode = DstNode.CreateNode()
                ' Copy all the stuff into the node
                NewDstNode.ImportFields(ChildSrcNode)
                NewDstNode.ProcessInfoDate = DateToStr(Date.Now)
                NewDstNode.ProcessInfoName = myProcessInfoName
                If ChildSrcNode.OtherLevel = OTHERLEVEL_INSTALATIONUNIT _
                    AndAlso ChildSrcNode.ContainerType = "" Then
                    NewDstNode.ContainerType = VisualFieldLabel("ContainerType.Other")
                    NewDstNode.ContainerCustomType = "Indefinido"
                End If
                NewDstNode.Upload()
                DstNode.AppendChild(NewDstNode)

                ' ********** cut here ********** 
                ' Remove this if you don't want to calculate complete unit id uppon import
                'DstNode.UpdateCompleteUnitID()
                'DstNode.Upload()
                'NewDstNode.UpdateCompleteUnitID()
                'NewDstNode.upload()
                ' ********** cut here ********** 

                ' recurse
                MergeTrees(NewDstNode, ChildSrcNode)
            Else ' Node exists... use that one
                Dim NewDstNode As LazyNode = Children.Item(Index)
                MergeTrees(NewDstNode, ChildSrcNode)
            End If
        Next

    End Sub


    Private Sub ImportFonds()
        Dim EADNode As LazyNode
        Try
            myProgressDialog.Message = ErrorMessage("ImportEAD.OpeningFile") & " " & Path.GetFileName(myFilename) & "..."
            EADNode = New EADLazyNode(myFilename)
            myProgressDialog.MaximumValue = EADNode.DescendantsCount()

            Dim FondsList As FondsListCollection = SQLLazyNode.GetFondsList( _
                                                        myServerAddress, _
                                                        myDatabase, _
                                                        myUsername, _
                                                        myPassword _
                                                    )

            Dim ExistingFondsItem As FondsListItem
            Dim DummySearchItem As New FondsListItem(0, EADNode.UnitId, "", Nothing, Nothing, False, 0)
            Dim Index As Integer = FondsList.IndexOf(DummySearchItem)

            If Index < 0 Then 'Fonds does not exists. Must create new fonds
                Dim SQLNode As SQLLazyNode = SQLLazyNode.CreateNewFonds( _
                                                            myServerAddress, _
                                                            myDatabase, _
                                                            myUsername, _
                                                            myPassword, _
                                                            EADNode.UnitId(), _
                                                            EADNode.UnitTitle(), _
                                                            myProcessInfoName _
                                                        )
                SQLNode.ImportFields(EADNode)
                SQLNode.ProcessInfoDate = DateToDB(Date.Now)
                SQLNode.ProcessInfoName = myProcessInfoName
                SQLNode.Upload()
                MergeTrees(SQLNode, EADNode)
                Logger.Append(myProcessInfoName, Logger.LogActions.ImportFonds, String.Format(LogMessage("Log.ImportedFonds"), EADNode.UnitId))
            Else
                ' The fonds already exists, get SQLNode and Merge
                ExistingFondsItem = FondsList.Item(Index)
                Dim SQLNode As SQLLazyNode = New SQLLazyNode( _
                                                        myServerAddress, _
                                                        myDatabase, _
                                                        myUsername, _
                                                        myPassword, _
                                                        ExistingFondsItem.ID _
                                                    )


                myProgressDialog.Hide()
                If MsgBox(String.Format(InfoMessage("ImportFonds.QuestionMergeFonds"), SQLNode.unitid), MsgBoxStyle.YesNo Or MsgBoxStyle.DefaultButton2 Or MsgBoxStyle.Question) = MsgBoxResult.Yes Then
                    myProgressDialog.Show()
                    MergeTrees(SQLNode, EADNode)
                    Logger.Append(myProcessInfoName, Logger.LogActions.ImportFonds, String.Format(LogMessage("Log.ImportedFonds"), EADNode.UnitId))
                End If
            End If
        Catch ex As SqlException
            Debug.WriteLine(ex.Message & ControlChars.NewLine & ex.StackTrace)
        Catch ex As Exception
            MsgBox(String.Format(ErrorMessage("ImportEAD.Cancelation"), Path.GetFileName(myFilename)), MsgBoxStyle.Critical)
            MsgBoxException(ex)
            'MsgBoxException(ex)
            'Debug.WriteLine(ex.Message & ControlChars.NewLine & ex.StackTrace)
        End Try

    End Sub

#End Region


#Region "Import Existing Fonds"

    Private Sub ImportExistingFonds()
        Cursor.Current = Cursors.WaitCursor
        Dim startTime As Date = Date.Now
        myProgressDialog = New ProgressDialog.ProgressDialog(True)
        myProgressDialog.MinimumValue = 0
        myProgressDialog.MaximumValue = mySourceFonds.DescendantsCount
        myProgressDialog.ShowTimeToEnd = True

        myProgressDialog.Text = String.Format(InfoMessage("Importer.Title"), Path.GetFileName(myFilename))
        '"A importar " & Path.GetFileName(myFilename) & "..."
        myProgressDialog.Show()

        Dim t As Thread = New Thread(New ThreadStart(AddressOf MergeFonds))
        't.IsBackground = True
        t.Start()

        While t.IsAlive
            System.Windows.Forms.Application.DoEvents()
        End While

        'ImportFonds()

        myProgressDialog.Hide()
        myProgressDialog.Close()
        myProgressDialog.Dispose()

        Dim finishTime As Date = Date.Now
        Dim timeSpan As timeSpan = finishTime.Subtract(startTime)

        Cursor.Current = Cursors.Default
    End Sub



    Private Sub MergeFonds()
        Try
            If myDestinationFonds Is Nothing OrElse mySourceFonds Is Nothing Then Exit Sub
            MergeTrees(myDestinationFonds, mySourceFonds)
        Catch ex As SqlException
            Debug.WriteLine(ex.Message & ControlChars.NewLine & ex.StackTrace)
        Catch ex As Exception
            MsgBox(String.Format(ErrorMessage("ImportEAD.Cancelation"), Path.GetFileName(myFilename)), MsgBoxStyle.Critical)
        End Try

    End Sub



#End Region


#Region "Import Tab Separated File"

    Private Sub ImportTabSeparatedFile()
        Cursor.Current = Cursors.WaitCursor

        myProgressDialog = New ProgressDialog.ProgressDialog(True)
        myProgressDialog.MinimumValue = 0
        myProgressDialog.MaximumValue = 20
        myProgressDialog.Text = String.Format(InfoMessage("Importer.Title"), Path.GetFileName(myFilename))
        myProgressDialog.Show()
        myProgressDialog.ShowTimeToEnd = True

        Dim t As Thread = New Thread(New ThreadStart(AddressOf ImportTabSeparatedFileIntoFonds))
        't.IsBackground = True
        t.Start()

        While t.IsAlive
            System.Windows.Forms.Application.DoEvents()
        End While

        'ImportFonds()

        myProgressDialog.Hide()
        myProgressDialog.Close()
        myProgressDialog.Dispose()


        Cursor.Current = Cursors.Default

    End Sub



    Private Sub ImportTabSeparatedFileIntoFonds()
        Dim TextFile As String = ReadTextFile(myFilename)
        Dim Lines As String() = TextFile.Split(vbNewLine)
        Dim HeaderFound As Boolean = False
        Dim Header As String()
        Dim Fields As String()
        myErrorCount = 0

        myProgressDialog.MaximumValue = Lines.Length

        For Each Line As String In Lines
            If Not Regex.IsMatch(Line, "^\n?$") Then ' ignore empty lines
                Debug.WriteLine("Reading " & Line)
                If Not HeaderFound Then ' read header
                    Header = Line.Split(ControlChars.Tab)
                    HeaderFound = True
                Else ' read rest of file
                    myProgressDialog.Message = Line.Replace(ControlChars.Tab, " ")
                    myProgressDialog.Increment()
                    Try
                        Fields = Line.Split(ControlChars.Tab)
                        Dim NewSQLNode As LazyNode = myDestinationFonds.CreateNode()

                        For i As Integer = 0 To Fields.Length - 1
                            CallByName(NewSQLNode, Header(i).Trim, CallType.Set, Fields(i).Trim)
                            NewSQLNode.UpdateProcessInfo(myProcessInfoName)
                            NewSQLNode.Upload()
                        Next
                        myDestinationFonds.AppendChild(NewSQLNode)
                    Catch ex As Exception
                        myErrorCount += 1
                        'MsgBoxException(ex)
                    End Try
                End If
            Else
                Debug.WriteLine("[Empty Line]")
            End If
        Next
    End Sub

#End Region


#Region "Import CALM Records"
    Private Sub ImportCALMRecords()
        Cursor.Current = Cursors.WaitCursor
        'Dim startTime As Date = Date.Now
        myProgressDialog = New ProgressDialog.ProgressDialog(True)
        myProgressDialog.MinimumValue = 0
        myProgressDialog.ShowTimeToEnd = True
        myProgressDialog.Text = String.Format(InfoMessage("Importer.Title"), Path.GetFileName(myFilename))

        myProgressDialog.Show()

        Dim t As Thread = New Thread(New ThreadStart(AddressOf ImportCALMRecordsThread))
        't.IsBackground = True
        t.Start()

        While t.IsAlive
            System.Windows.Forms.Application.DoEvents()
        End While

        'ImportFonds()

        myProgressDialog.Hide()
        myProgressDialog.Close()
        myProgressDialog.Dispose()

        ' Dim finishTime As Date = Date.Now
        'Dim timeSpan As TimeSpan = finishTime.Subtract(startTime)
        ' Debug.WriteLine(Path.GetFileName(myFilename) & ": " & timeSpan.Minutes.ToString.PadLeft(2, "0") & ":" & timeSpan.Seconds.ToString.PadLeft(2, "0") & " minutos")

        Cursor.Current = Cursors.Default
    End Sub



    Private Sub ImportCALMRecordsThread()
        Dim CALMNode As CALMLazyNode
        Try
            CALMNode = New CALMLazyNode(myFilename)

            myProgressDialog.Message = ErrorMessage("ImportEAD.OpeningFile") & " " & Path.GetFileName(myFilename) & "..."
            myProgressDialog.MaximumValue = CALMNode.DescendantsCount()

            Dim FondsList As FondsListCollection = SQLLazyNode.GetFondsList( _
                                                        myServerAddress, _
                                                        myDatabase, _
                                                        myUsername, _
                                                        myPassword _
                                                    )
            Dim ExistingFondsItem As FondsListItem
            Dim DummySearchItem As New FondsListItem(0, CALMNode.CompleteUnitId, "", Nothing, Nothing, False, 0)
            Dim Index As Integer = FondsList.IndexOf(DummySearchItem)

            ' Import digital objects into a new Digitization Project
            Dim DigitalAcquisitionProject As New DigitalAcquisitionProject(CALMNode.CompleteUnitId, "admin")

            ' ******************************************************

            'If Not FondsList.Contains(DummySearchItem) Then 'Fonds does not exists. Must create new fonds
            If Index < 0 Then 'Fonds does not exists. Must create new fonds
                Dim SQLNode As SQLLazyNode = SQLLazyNode.CreateNewFonds( _
                                                            myServerAddress, _
                                                            myDatabase, _
                                                            myUsername, _
                                                            myPassword, _
                                                            CALMNode.UnitId(), _
                                                            CALMNode.UnitTitle(), _
                                                            myProcessInfoName _
                                                        )




                Dim TmpDigitalObject As DigitArq.Data.Entities.MDA.DigitalObject = CALMNode.DigitalObject
                If Not TmpDigitalObject Is Nothing Then
                    TmpDigitalObject.ComponentID = CType(SQLNode.ID, Long)
                    DigitalAcquisitionProject.Add(TmpDigitalObject)
                End If

                SQLNode.ImportFields(CALMNode)
                SQLNode.OtherDescriptiveData = String.Format("Registo migrado a partir do sistema CALM em {0}.", DateToStr(Date.Today)) & vbNewLine & SQLNode.OtherDescriptiveData
                SQLNode.UpdateCompleteUnitID()
                SQLNode.Upload()

                'SQLNode.ProcessInfoDate = DateToDB(Date.Now)
                'SQLNode.ProcessInfoName = myProcessInfoName

                MergeCALMTrees(SQLNode, CALMNode, DigitalAcquisitionProject)

                ' Save Digital AcquisitionProject
                DigitalAcquisitionProject.Upload()

                Logger.Append(myProcessInfoName, Logger.LogActions.ImportFonds, String.Format(LogMessage("Log.ImportedFonds"), CALMNode.UnitId))
            Else  ' The fonds already exists, get SQLNode and Merge
                ExistingFondsItem = FondsList.Item(Index)
                Dim SQLNode As SQLLazyNode = New SQLLazyNode( _
                                                        myServerAddress, _
                                                        myDatabase, _
                                                        myUsername, _
                                                        myPassword, _
                                                        ExistingFondsItem.ID _
                                                    )


                myProgressDialog.Hide()
                If MsgBox(String.Format(InfoMessage("ImportFonds.QuestionMergeFonds"), SQLNode.unitid), MsgBoxStyle.YesNo Or MsgBoxStyle.DefaultButton2 Or MsgBoxStyle.Question) = MsgBoxResult.Yes Then
                    myProgressDialog.Show()

                    ' Import root node fields
                    SQLNode.MergeFields(CALMNode)
                    SQLNode.Upload()

                    MergeCALMTrees(SQLNode, CALMNode, DigitalAcquisitionProject)
                    Logger.Append(myProcessInfoName, Logger.LogActions.ImportFonds, String.Format(LogMessage("Log.ImportedFonds"), CALMNode.UnitId))

                    ' Save Digital AcquisitionProject
                    DigitalAcquisitionProject.Upload()
                End If
            End If
        Catch ex As Exception
            'MsgBox(String.Format(ErrorMessage("ImportEAD.Cancelation"), Path.GetFileName(myFilename)), MsgBoxStyle.Critical)
            MsgBoxException(ex)
            Debug.WriteLine(ex.Message & ControlChars.NewLine & ex.StackTrace)
        End Try

    End Sub



    ' This method imports nodes from CALM migrated XML records to digitarq
    ' It also imports digital objects
    Private Sub MergeCALMTrees(ByVal DstNode As LazyNode, ByVal SrcNode As CALMLazyNode, _
                               Optional ByVal DigitalAcquisitionProject As DigitalAcquisitionProject = Nothing)

        Dim Index As Integer
        Dim ChildSrcNode As CALMLazyNode
        Dim Children As LazyNodeCollection

        Children = DstNode.Children
        For Each ChildSrcNode In SrcNode.Children
            Try

                Index = Children.IndexOf(ChildSrcNode)
                myProgressDialog.Message = String.Format(InfoMessage("Importer.Progression"), """" & ChildSrcNode.UnitId & " - " & ChildSrcNode.UnitTitle & """")
                myProgressDialog.Increment()

                If myProgressDialog.Cancel Then Exit Sub

                'If Index < 0 OrElse ChildSrcNode.isLeaf Then ' node doesn't exist. Create new one

                If Index < 0 Then  ' node doesn't exist. Create new one
                    Dim NewDstNode As LazyNode = DstNode.CreateNode()
                    ' Copy all the stuff into the node
                    NewDstNode.ImportFields(ChildSrcNode)
                    'NewDstNode.ProcessInfoDate = DateToStr(Date.Now)
                    'NewDstNode.ProcessInfoName = myProcessInfoName

                    If ChildSrcNode.OtherLevel = OTHERLEVEL_INSTALATIONUNIT _
                        AndAlso ChildSrcNode.ContainerType = "" Then
                        NewDstNode.ContainerType = VisualFieldLabel("ContainerType.Other")
                        NewDstNode.ContainerCustomType = "Indefinido"
                    End If

                    NewDstNode.OtherDescriptiveData = String.Format("Registo migrado a partir do sistema CALM em {0}.", DateToStr(Date.Today)) & vbNewLine & NewDstNode.OtherDescriptiveData

                    NewDstNode.Upload()
                    DstNode.AppendChild(NewDstNode)


                    ' Import digital objects
                    Dim TmpDigitalObject As DigitArq.Data.Entities.MDA.DigitalObject = ChildSrcNode.DigitalObject
                    If Not TmpDigitalObject Is Nothing Then
                        TmpDigitalObject.ComponentID = CType(NewDstNode.ID, Long)
                        DigitalAcquisitionProject.Add(TmpDigitalObject)
                    End If


                    ' ********** cut here ********** 
                    ' Remove this if you don't want to calculate complete unit id uppon import
                    'DstNode.UpdateCompleteUnitID()
                    'DstNode.Upload()
                    'NewDstNode.UpdateCompleteUnitID()
                    'NewDstNode.upload()
                    ' ********** cut here ********** 

                    ' recurse
                    MergeCALMTrees(NewDstNode, ChildSrcNode, DigitalAcquisitionProject)
                Else ' Node exists... use that one
                    Dim NewDstNode As LazyNode = Children.Item(Index)
                    NewDstNode.MergeFields(ChildSrcNode)
                    NewDstNode.Upload()


                    ' Description object already exists, see if digital objects already exist. Create new digital object if not.
                    ' ********** cut here ********** 
                    ' Import digital objects
                    Dim TmpDigitalObject As DigitArq.Data.Entities.MDA.DigitalObject = ChildSrcNode.DigitalObject
                    If Not TmpDigitalObject Is Nothing AndAlso Not DigitalObjectExists(TmpDigitalObject.Name) Then
                        TmpDigitalObject.ComponentID = CType(NewDstNode.ID, Long)
                        DigitalAcquisitionProject.Add(TmpDigitalObject)
                    End If
                    ' ********** cut here ********** 

                    ' recurse
                    MergeCALMTrees(NewDstNode, ChildSrcNode, DigitalAcquisitionProject)
                End If
            Catch ex As Exception
                MsgBoxException(ex)
            End Try
        Next

    End Sub


    Private Function DigitalObjectExists(ByVal Name As String)
        Dim MGODDataLayer As New DigitArq.Data.Entities.MGOD.SqlCommands
        'Return MGODDataLayer.SelectDigitalObjects(String.Format("SELECT * FROM DigitalObjects WHERE Name = '{0}'", Name), False).Count > 0
        Return MGODDataLayer.SelectDigitalObjects(Name).Count > 0
    End Function

#End Region


#Region "Import DigitArq Interoperability Format"


    Private Sub ImportDIF()
        Cursor.Current = Cursors.WaitCursor
        'Dim startTime As Date = Date.Now
        myProgressDialog = New ProgressDialog.ProgressDialog(True)
        myProgressDialog.MinimumValue = 0
        myProgressDialog.ShowTimeToEnd = True
        myProgressDialog.Text = String.Format(InfoMessage("Importer.Title"), Path.GetFileName(myFilename))

        myProgressDialog.Show()

        Dim t As Thread = New Thread(New ThreadStart(AddressOf ImportDIFThreadEventDriven))
        t.Start()

        While t.IsAlive
            System.Windows.Forms.Application.DoEvents()
        End While


        myProgressDialog.Hide()
        myProgressDialog.Close()
        myProgressDialog.Dispose()

        ' Dim finishTime As Date = Date.Now
        'Dim timeSpan As TimeSpan = finishTime.Subtract(startTime)
        ' Debug.WriteLine(Path.GetFileName(myFilename) & ": " & timeSpan.Minutes.ToString.PadLeft(2, "0") & ":" & timeSpan.Seconds.ToString.PadLeft(2, "0") & " minutos")

        Cursor.Current = Cursors.Default
    End Sub





#Region "Event driven XML processing"

    'Private Sub ImportDIFThreadEventDriven()
    '    Try

    '        File.Delete(DIF_IMPORTER_LOG_PATH)

    '        myProgressDialog.MaximumValue = CountRecords()
    '        Dim FirstRecordMap As Hashtable = GetFirstRecord()
    '        If FirstRecordMap Is Nothing Then Exit Sub

    '        If "F".Equals(CStr(FirstRecordMap.Item("OtherLevel"))) Then 'contains fonds record
    '            ' Dim response As MsgBoxResult = MsgBox(InfoMessage("ImportFonds.MergeFonds"), MsgBoxStyle.Question Or MsgBoxStyle.YesNo)
    '            Dim ParentFondsID As Integer = InsertRecord(FirstRecordMap, Nothing)
    '            ImportRecords(ParentFondsID)
    '        Else ' does not contain fonds record
    '            myProgressDialog.Hide()
    '            Dim response As MsgBoxResult = MsgBox(InfoMessage("ImportFonds.PartialFondsImport"), MsgBoxStyle.Question Or MsgBoxStyle.YesNo)
    '            If response = MsgBoxResult.No Then Exit Sub
    '            myProgressDialog.Show()
    '            Dim ParentCompleteUnitID As String = GetRecordParentReference(FirstRecordMap)
    '            Dim ParentFondsID As Integer = GetParentFondsID(ParentCompleteUnitID)
    '            If (parentFondsID >= 0) Then
    '                ImportRecords(ParentFondsID)
    '            Else
    '                MsgBox(InfoMessage("ImportFonds.CannotImportDIF"), MsgBoxStyle.Critical)
    '            End If

    '            myProgressDialog.Show()
    '        End If


    '    Catch ex As Exception
    '        MsgBoxException(ex)
    '        Debug.WriteLine(ex.Message & ControlChars.NewLine & ex.StackTrace)
    '    End Try
    'End Sub

    Private Sub ImportDIFThreadEventDriven()
        Try

            File.Delete(DIF_IMPORTER_LOG_PATH)

            myProgressDialog.Hide()
            myProgressDialog.MaximumValue = CountRecords()
            Dim FirstRecordMap As Hashtable = GetFirstRecord()
            If FirstRecordMap Is Nothing Then Exit Sub


            Dim FirstRecordCompleteUnitID As String = CStr(FirstRecordMap.Item("CompleteUnitId"))



            If "F".Equals(CStr(FirstRecordMap.Item("OtherLevel"))) Then 'contains fonds record
                If FondsAlreadyExists(FirstRecordCompleteUnitID) Then
                    Dim response As MsgBoxResult = MsgBox(String.Format(InfoMessage("ImportFonds.MergeFonds"), FirstRecordCompleteUnitID), MsgBoxStyle.Question Or MsgBoxStyle.YesNo)
                    myProgressDialog.Show()
                    If response = MsgBoxResult.Yes Then
                        Debug.WriteLine("Merging with existing fonds...")
                        Dim ExistingFondsID As Integer = GetParentFondsID(FirstRecordCompleteUnitID)
                        Dim NewFondsID As Integer = InsertRecord(FirstRecordMap, Nothing)
                        MergeRecords(ExistingFondsID, NewFondsID)
                        ImportRecords(ExistingFondsID, True)
                    Else
                        Debug.WriteLine("Creating a new fonds...")
                        Dim ParentFondsID As Integer = InsertRecord(FirstRecordMap, Nothing)
                        ImportRecords(ParentFondsID)
                    End If
                Else
                    myProgressDialog.Show()
                    Dim ParentFondsID As Integer = InsertRecord(FirstRecordMap, Nothing)
                    ImportRecords(ParentFondsID)
                End If

            Else ' does not contain fonds record
                If FondsAlreadyExists(FirstRecordCompleteUnitID) Then
                    Dim response As MsgBoxResult
                    response = MsgBox(String.Format(InfoMessage("ImportFonds.MergeFonds"), FirstRecordCompleteUnitID), MsgBoxStyle.Question Or MsgBoxStyle.YesNo)
                    myProgressDialog.Show()
                    If response = MsgBoxResult.Yes Then
                        Debug.WriteLine("Merging with existing fonds...")
                        Dim ExistingFondsID As Integer = GetParentFondsID(FirstRecordCompleteUnitID)
                        Dim NewFondsID As Integer = InsertRecord(FirstRecordMap, Nothing)
                        MergeRecords(ExistingFondsID, NewFondsID)
                        ImportRecords(ExistingFondsID, True)
                    Else
                        response = MsgBox(InfoMessage("ImportFonds.PartialFondsImport"), MsgBoxStyle.Question Or MsgBoxStyle.YesNo)
                        If response = MsgBoxResult.No Then Exit Sub
                        myProgressDialog.Show()
                        Dim ParentCompleteUnitID As String = GetRecordParentReference(FirstRecordMap)
                        Dim ParentFondsID As Integer = GetParentFondsID(ParentCompleteUnitID)
                        If (ParentFondsID >= 0) Then
                            ImportRecords(ParentFondsID)
                        Else
                            MsgBox(InfoMessage("ImportFonds.CannotImportDIF"), MsgBoxStyle.Critical)
                        End If
                    End If
                End If
            End If
        Catch ex As Exception
            MsgBoxException(ex)
            Debug.WriteLine(ex.Message & ControlChars.NewLine & ex.StackTrace)
        End Try
    End Sub


    Function FondsAlreadyExists(ByVal CompleteUnitID As String) As Boolean
        Try
            If GetParentFondsID(CompleteUnitID) > 0 Then
                Return True
            Else
                Return False
            End If
        Catch ex As Exception
            Return False
        End Try

    End Function

    Private Function GetParentFondsID(ByVal CompleteUnitID As String) As Integer
        Dim ComponentID As Integer = SQLLazyNode.GetComponentID(myServerAddress, myDatabase, myUsername, myPassword, CompleteUnitID)
        Dim Node As SQLLazyNode = New SQLLazyNode(myServerAddress, myDatabase, myUsername, myPassword, ComponentID)
        Return Node.ParentFondsID
    End Function


    Private Function GetFirstRecord() As Hashtable
        Debug.WriteLine("Getting First Record in DIF")
        Dim NumberOfRecords As Integer = 0

        Dim Result As Hashtable = Nothing

        Dim xmlNameTable As xmlNameTable = New NameTable
        xmlNameTable.Add("record")

        Dim xmlTextReader As xmlTextReader = New xmlTextReader(myFilename, xmlNameTable)
        xmlTextReader.WhitespaceHandling() = WhitespaceHandling.Significant

        Dim reader As New XmlValidatingReader(xmlTextReader)
        reader.ValidationType = ValidationType.None
        reader.EntityHandling = EntityHandling.ExpandEntities

        reader.MoveToContent()
        reader.Read()
        While Not reader.EOF
            If reader.NodeType = XmlNodeType.Element AndAlso reader.Name.Equals("record") Then
                NumberOfRecords += 1
                Dim RecordAsXML As String = reader.ReadOuterXml
                Dim RecordMap As Hashtable = XMLToRecordMap(RecordAsXML)
                Debug.WriteLine("Processing UnitId = " & CStr(RecordMap.Item("CompleteUnitId")) & " Level = " & CStr(RecordMap.Item("OtherLevel")))
                Result = RecordMap
                Exit While
            Else
                reader.Read()
            End If
            If (myProgressDialog.Cancel = True) Then Exit Function
        End While
        xmlTextReader.Close()
        reader.Close()
        Return Result
    End Function


    Private Function CountRecords() As Integer
        Debug.WriteLine("Counting Records in DIF")
        Dim NumberOfRecords As Integer = 0

        Dim xmlNameTable As xmlNameTable = New NameTable
        xmlNameTable.Add("record")

        Dim xmlTextReader As xmlTextReader = New xmlTextReader(myFilename, xmlNameTable)
        xmlTextReader.WhitespaceHandling() = WhitespaceHandling.Significant

        Dim reader As New XmlValidatingReader(xmlTextReader)
        reader.ValidationType = ValidationType.None
        reader.EntityHandling = EntityHandling.ExpandEntities

        reader.MoveToContent()
        reader.Read()
        While Not reader.EOF
            If reader.NodeType = XmlNodeType.Element AndAlso reader.Name.Equals("record") Then
                reader.ReadOuterXml()
                NumberOfRecords += 1
            Else
                reader.Read()
            End If
            If (myProgressDialog.Cancel = True) Then Exit Function
        End While
        xmlTextReader.Close()
        reader.Close()
        Return NumberOfRecords
    End Function



    Private Sub ImportRecords(ByVal ParentFondsID As Integer, Optional ByVal MergeFields As Boolean = False)
        Dim xmlNameTable As xmlNameTable = New NameTable
        xmlNameTable.Add("record")

        Dim xmlTextReader As xmlTextReader = New xmlTextReader(myFilename, xmlNameTable)
        xmlTextReader.WhitespaceHandling() = WhitespaceHandling.Significant

        Dim reader As New XmlValidatingReader(xmlTextReader)
        reader.ValidationType = ValidationType.None
        reader.EntityHandling = EntityHandling.ExpandEntities

        reader.MoveToContent()
        While Not reader.EOF
            If reader.NodeType = XmlNodeType.Element AndAlso reader.Name.Equals("record") Then
                Dim RecordAsXML As String = reader.ReadOuterXml
                Dim RecordMap As Hashtable = XMLToRecordMap(RecordAsXML)
                Debug.WriteLine("Importing UnitId = " & CStr(RecordMap.Item("CompleteUnitId")))
                If Not "F".Equals(CStr(RecordMap.Item("OtherLevel"))) Then
                    RecordMap.Add("ParentFondsID", CStr(ParentFondsID))
                    InsertRecord(RecordMap, ParentFondsID, MergeFields)
                End If
                myProgressDialog.Increment()
            Else
                reader.Read()
            End If
            If (myProgressDialog.Cancel = True) Then Exit Sub
        End While
        xmlTextReader.Close()
        reader.Close()
    End Sub


    Function XMLToRecordMap(ByVal RecordAsXML As String) As Hashtable
        Try
            Dim XMLDocument As New XMLDocument
            XMLDocument.LoadXml(RecordAsXML)
            Dim Record As XmlNode = XMLDocument.SelectSingleNode("/record")
            Dim RecordMap As Hashtable = XMLToRecordMap(Record)
            Return RecordMap
        Catch ex As Exception
            logImportIssue("- Erro ao importar o registo: " & RecordAsXML)
            Return New Hashtable
        End Try
    End Function

    Private Sub logImportIssue(ByVal message As String)
        Try
            appendToTextFile(DIF_IMPORTER_LOG_PATH, message)
        Catch ex As Exception
            Debug.Write("Cannot create file " & DIF_IMPORTER_LOG_PATH)
        End Try

    End Sub


    Private Function XMLToRecordMap(ByRef Record As XmlNode) As Hashtable
        Dim RecordMap As New Hashtable

        If Record Is Nothing Then Return Nothing

        For Each Field As XmlNode In Record.SelectNodes("field")
            Dim ColumnName As String = Field.SelectSingleNode("@name").Value
            Dim ColumnType As String = Field.SelectSingleNode("@type").Value
            Dim ColumnValue As String = Field.InnerText
            If (ColumnType.Equals("float")) Then ColumnValue.Replace(",", ".")

            ' only add non ID elements
            If (Not ColumnName.Equals("ID") AndAlso _
                Not ColumnName.Equals("ParentID") AndAlso _
                Not ColumnName.Equals("ParentFondsID")) Then
                RecordMap.Add(ColumnName, ColumnValue)
            End If
        Next

        Return RecordMap
    End Function


    Private Sub InsertRelatedRecords(ByRef RelatedRecords As XmlNodeList, ByVal TableName As String, ByVal ComponentID As String)
        For Each RelatedRecord As XmlNode In RelatedRecords
            Dim RelatedRecordMap As Hashtable = XMLToRecordMap(RelatedRecord)
            If (RelatedRecordMap.ContainsKey("ComponentID")) Then RelatedRecordMap.Remove("ComponentID")
            If (RelatedRecordMap.ContainsKey("ID")) Then RelatedRecordMap.Remove("ID")
            If (RelatedRecordMap.ContainsKey("NoteID")) Then RelatedRecordMap.Remove("NoteID")
            RelatedRecordMap.Add("ComponentID", ComponentID)

            InsertRelatedRecord(RelatedRecordMap, TableName)
        Next

    End Sub


    Private Function InsertRelatedRecord(ByRef RelatedRecord As Hashtable, ByVal TableName As String) As Long
        Dim SQLConnection As SQLConnection = New SQLConnection
        Dim SQLCommand As New SQLCommand
        Dim Transaction As SqlTransaction

        SQLConnection.ConnectionString = String.Format("Data Source={0}; Initial Catalog={1}; User ID={2}; Password={3}", myServerAddress, myDatabase, myUsername, myPassword)
        SQLCommand.Connection = SQLConnection


        Dim ColumnNames As String = GetColumnNames(RelatedRecord)
        Dim ColumnValues As String = GetColumnValues(RelatedRecord)
        Dim NewID As Long = -1

        Try
            SQLConnection.Open()
            Transaction = SQLConnection.BeginTransaction(IsolationLevel.ReadCommitted)
            SQLCommand.Transaction = Transaction

            SQLCommand.CommandText = String.Format("INSERT INTO {0} ({1}) VALUES ({2})", TableName, ColumnNames, ColumnValues)
            SQLCommand.ExecuteNonQuery()
            SQLCommand.CommandText = "SELECT SCOPE_IDENTITY() AS ID"
            NewID = CInt(SQLCommand.ExecuteScalar().ToString)

            Transaction.Commit()
        Catch ex As SqlException
            Debug.WriteLine("SQLNode.Update " & ex.Message)
            If Not ex.Number = 8152 Then
                Transaction.Rollback()
                MsgBoxSqlException(ex)
            End If
        Catch ex As Exception
            Debug.WriteLine("Importer.InsertRecord " & ex.Message)
            MsgBoxException(ex)
        Finally
            SQLConnection.Close()
        End Try
        Return NewID
    End Function


    Private Function InsertRecord(ByRef Record As Hashtable, ByVal ParentFondsID As String, Optional ByVal MergeFields As Boolean = False) As Integer
        Dim SQLConnection As SQLConnection = New SQLConnection
        Dim SQLCommand As New SQLCommand
        Dim Transaction As SqlTransaction

        SQLConnection.ConnectionString = String.Format("Data Source={0}; Initial Catalog={1}; User ID={2}; Password={3}", myServerAddress, myDatabase, myUsername, myPassword)
        SQLCommand.Connection = SQLConnection

        Dim ExistingComponentID As Integer = SQLLazyNode.GetComponentID(myServerAddress, myDatabase, myUsername, myPassword, Record.Item("CompleteUnitId"), ParentFondsID)

        ' Calculate parent reference
        Dim ParentReference As String = GetRecordParentReference(Record)
        Dim ParentID As String = CStr(SQLLazyNode.GetComponentID(myServerAddress, myDatabase, myUsername, myPassword, ParentReference, ParentFondsID))

        If (ParentID.Equals("-1")) Then ParentID = "NULL"

        If (ParentID = "NULL" AndAlso Record.Item("OtherLevel") <> "F") Then
            ' Record has no parent and should be discarded.
            logImportIssue("- Registo sem ascendente: " & Record.Item("CompleteUnitId"))
            Exit Function
        End If

        Record.Add("ParentID", ParentID)

        Dim ColumnNames As String = GetColumnNames(Record)
        Dim ColumnValues As String = GetColumnValues(Record)
        Dim NewID As Integer = -1

        Try
            SQLConnection.Open()
            Transaction = SQLConnection.BeginTransaction(IsolationLevel.ReadCommitted)
            SQLCommand.Transaction = Transaction

            SQLCommand.CommandText = String.Format("INSERT INTO Components ({0}) VALUES ({1})", ColumnNames, ColumnValues)
            SQLCommand.ExecuteNonQuery()
            SQLCommand.CommandText = "SELECT SCOPE_IDENTITY() AS ID"
            NewID = CInt(SQLCommand.ExecuteScalar().ToString)

            'Only happens when record level is fonds
            If (ParentFondsID Is Nothing) Then
                SQLCommand.CommandText = String.Format("UPDATE Components SET ParentFondsID={0} WHERE ID={0}", NewID)
                SQLCommand.ExecuteNonQuery()
            End If

            Transaction.Commit()
        Catch ex As SqlException
            Debug.WriteLine("SQLNode.Update " & ex.Message)
            If Not ex.Number = 8152 Then
                Transaction.Rollback()
                MsgBoxSqlException(ex)
            End If
        Catch ex As Exception
            Debug.WriteLine("Importer.InsertRecord " & ex.Message)
            logImportIssue("- Erro de importao: " & ex.Message)
            MsgBoxException(ex)
        Finally
            SQLConnection.Close()
        End Try



        ' Record already existed in the database. Merging the two records
        If MergeFields AndAlso ExistingComponentID > 0 Then
            MergeRecords(ExistingComponentID, NewID)
        End If

        Return NewID
    End Function

    Private Sub MergeRecords(ByVal ExistingComponentID As Integer, ByVal MergeFromComponentID As Integer)
        Dim OldNode As LazyNode = New SQLLazyNode(myServerAddress, myDatabase, myUsername, myPassword, ExistingComponentID)
        Dim NewNode As LazyNode = New SQLLazyNode(myServerAddress, myDatabase, myUsername, myPassword, MergeFromComponentID)

        If (OldNode Is Nothing OrElse NewNode Is Nothing) Then
            Debug.WriteLine("MergeRecords: One of the nodes is nothing! Exiting...")
            Exit Sub
        End If
        Debug.WriteLine(String.Format("Node {0} already exists with ID {1}", NewNode.ID, OldNode.ID))

        OldNode.MergeFields(NewNode)
        OldNode.Upload()
        SQLLazyNode.RemoveNode(myServerAddress, myDatabase, myUsername, myPassword, MergeFromComponentID)

    End Sub



    Private Function GetColumnNames(ByVal Record As Hashtable) As String
        Dim keys As ICollection = Record.Keys
        Dim ret As String = ""
        For Each key As String In keys
            Dim value As String = Record.Item(key)
            If value.Length > 0 Then
                ret += key & ","
            End If
        Next
        Return ret.Substring(0, ret.Length - 1)
    End Function


    Private Function GetColumnValues(ByVal Record As Hashtable) As String
        Dim keys As ICollection = Record.Keys
        Dim ret As String = ""
        For Each key As String In keys
            Dim value As String = Record.Item(key)
            If value.Length > 0 Then

                ' handle nulls differently
                If value.ToUpper.Equals("NULL") Then
                    ret += String.Format("{0},", StrToDB(value))
                ElseIf key = "ExtentMl" Then
                    ret += String.Format("{0},", StrToDB(value).Replace(",", "."))
                Else
                    ret += String.Format("'{0}',", StrToDB(value))
                End If
            End If

        Next
        Return ret.Substring(0, ret.Length - 1)
    End Function


    Private Function GetRecordParentReference(ByVal Record As Hashtable) As String
        Dim CompleteUnitID As String = CStr(Record.Item("CompleteUnitId"))
        If CompleteUnitID Is Nothing Then Return Nothing

        Dim UnitID As String = CStr(Record.Item("UnitId"))
        Dim OtherLevel As String = CStr(Record.Item("OtherLevel"))

        'is fonds, has no parent
        If (OTHERLEVEL_FONDS.Equals(OtherLevel)) Then
            Return ""
        End If

        Dim ParentCompleteUnitID As String = CompleteUnitID.Substring(0, CompleteUnitID.Length - UnitID.Length - 1)
        Return ParentCompleteUnitID
    End Function



#End Region

#Region "Deprecated DIF import functions"

    'Private Sub ImportRecord(ByVal RecordAsXML As String, ByVal ParentFondsID As Integer)
    '    Debug.WriteLine("Inserting non-fonds Record " & RecordAsXML)

    '    Dim XMLDocument As New XmlDocument
    '    XMLDocument.LoadXml(RecordAsXML)


    '    ' non fonds records
    '    Dim Record As XmlNode = XMLDocument.SelectSingleNode("/record")
    '    Dim RecordMap As Hashtable = XMLToRecordMap(Record)

    '    If (RecordMap Is Nothing) Then Exit Sub

    '    Dim OtherLevel As String = CStr(RecordMap.Item("OtherLevel"))
    '    If OtherLevel Is Nothing OrElse OtherLevel = "" OrElse OtherLevel = "F" Then
    '        Debug.WriteLine("Invalid level. Cannot be processed!")
    '        Exit Sub
    '    End If

    '    Debug.WriteLine("Processing " & CStr(RecordMap.Item("OtherLevel") & " - " & CStr(RecordMap.Item("UnitTitle"))))


    '    '******************************
    '    Dim CompleteUnitID As String = RecordMap.Item("CompleteUnitId")
    '    Dim UnitTitle As String = RecordMap.Item("UnitTitle")
    '    myProgressDialog.Message = String.Format(InfoMessage("Importer.Progression"), """" & CompleteUnitID & " - " & UnitTitle & """")

    '    RecordMap.Add("ParentFondsID", CStr(ParentFondsID))
    '    Dim ComponentID As Long = InsertRecord(RecordMap, ParentFondsID)

    '    InsertRelatedRecords(Record.SelectNodes("bibliography/record"), "bibliography", CStr(ComponentID))
    '    InsertRelatedRecords(Record.SelectNodes("archivistnotes/record"), "archivistnotes", CStr(ComponentID))
    '    InsertRelatedRecords(Record.SelectNodes("chronlist/record"), "chronlist", CStr(ComponentID))
    '    InsertRelatedRecords(Record.SelectNodes("daogrp/record"), "daogrp", CStr(ComponentID))
    '    '******************************

    '    XMLDocument = Nothing
    'End Sub
    'Private Sub ImportDIFThread()
    '    Try
    '        Dim XmlDocument As XmlDocument

    '        Dim vr As XmlValidatingReader = New XmlValidatingReader(New XmlTextReader(myFilename))
    '        vr.ValidationType = ValidationType.None
    '        vr.EntityHandling = EntityHandling.ExpandEntities

    '        XmlDocument = New XmlDocument
    '        'XmlDocument.Load(myFilename)
    '        XmlDocument.Load(vr)


    '        Debug.WriteLine("Inserting Fonds Record")
    '        Dim FondsRecord As XmlNode = XmlDocument.SelectSingleNode("/dif/record[field/@name='OtherLevel' and field/text()='F']")
    '        Debug.WriteLine(FondsRecord.InnerXml)

    '        Dim fondsRecordMap As Hashtable = XMLToRecordMap(FondsRecord)
    '        Dim ParentFondsID As Long = InsertRecord(fondsRecordMap, Nothing)

    '        ' add fonds related records
    '        InsertRelatedRecords(FondsRecord.SelectNodes("bibliography/record"), "bibliography", CStr(ParentFondsID))
    '        InsertRelatedRecords(FondsRecord.SelectNodes("archivistnotes/record"), "archivistnotes", CStr(ParentFondsID))
    '        InsertRelatedRecords(FondsRecord.SelectNodes("chronlist/record"), "chronlist", CStr(ParentFondsID))
    '        InsertRelatedRecords(FondsRecord.SelectNodes("daogrp/record"), "daogrp", CStr(ParentFondsID))


    '        ' non fonds records
    '        Dim RecordsList As XmlNodeList = XmlDocument.SelectNodes("/dif/record[field/@name='OtherLevel' and not(field/text()='F')]")
    '        myProgressDialog.MaximumValue = RecordsList.Count + 1

    '        For Each Record As XmlNode In RecordsList
    '            Dim RecordMap As Hashtable = XMLToRecordMap(Record)
    '            Dim CompleteUnitID As String = RecordMap.Item("CompleteUnitId")
    '            Dim UnitTitle As String = RecordMap.Item("UnitTitle")
    '            myProgressDialog.Message = String.Format(InfoMessage("Importer.Progression"), """" & CompleteUnitID & " - " & UnitTitle & """")

    '            RecordMap.Add("ParentFondsID", CStr(ParentFondsID))
    '            Dim ComponentID As Long = InsertRecord(RecordMap, ParentFondsID)

    '            InsertRelatedRecords(Record.SelectNodes("bibliography/record"), "bibliography", CStr(ComponentID))
    '            InsertRelatedRecords(Record.SelectNodes("archivistnotes/record"), "archivistnotes", CStr(ComponentID))
    '            InsertRelatedRecords(Record.SelectNodes("chronlist/record"), "chronlist", CStr(ComponentID))
    '            InsertRelatedRecords(Record.SelectNodes("daogrp/record"), "daogrp", CStr(ComponentID))

    '            myProgressDialog.Increment()
    '            If (myProgressDialog.Cancel = True) Then Exit For
    '        Next
    '    Catch ex As Exception
    '        MsgBoxException(ex)
    '        Debug.WriteLine(ex.Message & ControlChars.NewLine & ex.StackTrace)
    '    End Try
    'End Sub
#End Region

#End Region


End Class

