Imports System.Data.SqlClient
Imports System.IO
Imports System.Configuration
Imports DigitArq.Data.Entities.MGOD
Imports DigitArq.Definitions.MGOD

Public Class ReportDataSetBuilder

#Region "Data base connection"

    Private SQLServerSettings As Hashtable = ConfigurationSettings.GetConfig("sqlServerSettings")
    Private ServerAddress As String = SQLServerSettings("ServerAddress")
    Private Database As String = SQLServerSettings("Database")
    Private Username As String = SQLServerSettings("Username")
    Private Password As String = SQLServerSettings("Password")
    Private ConnectTimeout As String = SQLServerSettings("ConnectTimeout")

    Private ConnectionString As String = String.Format("Data Source={0}; Initial Catalog={1}; User ID={2}; password={3}; Connect Timeout={4};", ServerAddress, Database, Username, Password, ConnectTimeout)

#End Region

#Region "Project reports"

    'Generate digitalization projects report data
    Public Function GenerateDigitalizationProjectsReportData(ByVal LVDOCollection As ListView.SelectedListViewItemCollection) As ProjectsData
        Dim myData As New ProjectsData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Dim strProjectsIDList As String = GetProjectIDList(LVDOCollection)
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = String.Format("SELECT ProjectID, Name, UserName, CreationDateTime " & _
                        "FROM Projects " & _
                        "WHERE ProjectID IN {0};", strProjectsIDList)

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "Projects")

            myCommand = String.Format("SELECT DO.DigitalObjectID, ProjectID, Name, ArchiveDateTime, ISNULL(F.QuantityOfTerminalObjects,0) AS QuantityOfTerminalObjects, Username " & _
                        "FROM DigitalObjects DO " & _
                        "LEFT JOIN " & _
                        "(SELECT DigitalObjectID, COUNT(DigitalObjectID) AS QuantityOfTerminalObjects " & _
                        "FROM Files " & _
                        "GROUP BY DigitalObjectID) AS F " & _
                        "ON DO.DigitalObjectID=F.DigitalObjectID " & _
                        "WHERE ProjectID IN {0};", strProjectsIDList)

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DigitalObjects")

        Catch e As Exception
            Throw (e)
        Finally
            myReader.Dispose()
        End Try
        Return myData
    End Function

    'Generate operators report data
    Public Function GenerateDOOperatorsReportData(ByVal LVDOCollection As ListView.SelectedListViewItemCollection) As DOOperatorsData
        Dim myData As New DOOperatorsData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Dim strProjectIDList As String = GetProjectIDList(LVDOCollection)
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = String.Format("SELECT DO.DigitalObjectID, ProjectID, Name, ArchiveDateTime, F.QuantityOfTerminalObjects, Username " & _
                        "FROM DigitalObjects DO " & _
                        "INNER JOIN " & _
                        "(SELECT DigitalObjectID, COUNT(DigitalObjectID) AS QuantityOfTerminalObjects " & _
                        "FROM Files " & _
                        "GROUP BY DigitalObjectID) AS F " & _
                        "ON DO.DigitalObjectID=F.DigitalObjectID " & _
                        "WHERE ProjectID IN {0} " & _
                        "ORDER BY ArchiveDateTime ASC;", strProjectIDList)

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DigitalObjects")

            myCommand = String.Format("SELECT DISTINCT ID, CompleteUnitID " & _
                        "FROM Components C " & _
                        "INNER JOIN DaoGrp DG ON C.ID=DG.ComponentID " & _
                        "INNER JOIN DigitalObjects DO ON DG.DigitalObjectID=DO.DigitalObjectID " & _
                        "WHERE DO.ProjectID IN {0};", strProjectIDList)

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "Components")

            myCommand = String.Format("SELECT DISTINCT DG.DigitalObjectID, DG.ComponentID " & _
                        "FROM DigitalObjects DO " & _
                        "INNER JOIN DaoGrp DG ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                        "WHERE DO.ProjectID IN {0};", strProjectIDList)

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DaoGrp")

        Catch e As Exception
            Throw (e)
        Finally
            myReader.Dispose()
        End Try
        Return myData
    End Function

#End Region

#Region "Digital objects reports"

    'Generate digitalization profiles report data
    Public Function GenerateDPReportData() As DigitalizationProfilesData
        Dim myData As New DigitalizationProfilesData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = "SELECT * FROM DigitalizationProfile ORDER BY Name"

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DigitalizationProfiles")

        Catch e As SqlException
            Throw (e)
        Finally
            myReader.Dispose()
        End Try
        Return myData
    End Function


    'Generate DO revision report data
    Public Function GenerateDORevisionReportData(ByVal DigitalObjectCollection As DigitalObjectCollection) As DORevisionData
        Dim myData As New DORevisionData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Dim strDigitalObjectIDList As String = GetDigitalObjectIDList(DigitalObjectCollection)
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = String.Format("SELECT DO.DigitalObjectID, ProjectID, Name, ArchiveID, RevisionDateTime, ISNULL(F.QuantityOfTerminalObjects,0) AS QuantityOfTerminalObjects, TopographicalQuota " & _
                        "FROM DigitalObjects DO " & _
                        "LEFT JOIN " & _
                        "(SELECT DigitalObjectID, COUNT(DigitalObjectID) AS QuantityOfTerminalObjects " & _
                        "FROM Files " & _
                        "GROUP BY DigitalObjectID) AS F " & _
                        "ON DO.DigitalObjectID=F.DigitalObjectID " & _
                        "WHERE DO.DigitalObjectID IN {0} " & _
                        "ORDER BY ArchiveDateTime ASC;", strDigitalObjectIDList)

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DigitalObjects")

        Catch e As Exception
            Throw (e)
        Finally
            myReader.Dispose()
        End Try
        Return myData
    End Function


    'Generate metadata of digital objects report data
    Public Function GenerateMIDigitalObjectsReportData(ByVal DigitalObjectCollection As DigitalObjectCollection) As DigitalObjectsData
        Dim myData As New DigitalObjectsData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = String.Format("SELECT DO.DigitalObjectID, Name, ArchiveID, CaptureEntityCorporate, " & _
                        "ArchiveDateTime, CreationDateTime, DepositDateTime, ISNULL(F.QuantityOfTerminalObjects, 0) AS QuantityOfTerminalObjects, Username, TopographicalQuota " & _
                        "FROM DigitalObjects DO " & _
                        "LEFT JOIN " & _
                        "(SELECT DigitalObjectID, COUNT(DigitalObjectID) AS QuantityOfTerminalObjects " & _
                        "FROM Files " & _
                        "GROUP BY DigitalObjectID) AS F " & _
                        "ON DO.DigitalObjectID=F.DigitalObjectID " & _
                        "WHERE DO.DigitalObjectID IN {0};", GetDigitalObjectIDList(DigitalObjectCollection))

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DigitalObjects")

        Catch e As Exception
            Throw (e)
        Finally
            myReader.Dispose()
        End Try
        Return myData
    End Function


    'Generate metadata of DO files report data
    Public Function GenerateMIImagesReportData(ByVal DigitalObjectCollection As DigitalObjectCollection) As DOImagesData
        Dim myData As New DOImagesData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String

        Dim strDigitalObjectIDList As String = GetDigitalObjectIDList(DigitalObjectCollection)
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = String.Format("SELECT DO.DigitalObjectID, Name AS DOName, ArchiveID, ArchiveDateTime, " & _
                        "ISNULL(F.QuantityOfTerminalObjects, 0) AS QuantityOfTerminalObjects, TopographicalQuota, Username " & _
                        "FROM DigitalObjects DO " & _
                        "LEFT JOIN " & _
                        "(SELECT DigitalObjectID, COUNT(DigitalObjectID) AS QuantityOfTerminalObjects " & _
                        "FROM Files " & _
                        "GROUP BY DigitalObjectID) AS F " & _
                        "ON DO.DigitalObjectID=F.DigitalObjectID " & _
                        "WHERE DO.DigitalObjectID IN {0};", GetDigitalObjectIDList(DigitalObjectCollection))

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DigitalObjects")

            myCommand = "SELECT DOF.FileID, DOF.DigitalObjectID, DOF.Name, DOF.Compression, DOF.ColorSpace, DOF.LampTypeID, DOF.Resolution, DOF.ImageHeight, DOF.ImageWidth, DOF.BitDepth, DOF.MIMEType, DOF.ImageSize, TP.DeviceSource, TP.ScannerManufacturer, TP.ScannerModelName, TP.ScanningSoftware, TP.ScanningSoftwareVersionNo, TP.OperatingSystem, TP.OperatingSystem " & _
                        "FROM DOFiles DOF INNER JOIN DigitalObjects DO ON DOF.DigitalObjectID=DO.DigitalObjectID " & _
                        "INNER JOIN TechnologicalPlatform TP ON DOF.TplatformID=TP.TplatformID " & _
                        "WHERE DO.DigitalObjectID IN " & strDigitalObjectIDList

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DOFiles")

            myCommand = "SELECT DISTINCT C.ID, C.Otherlevel, C.CompleteUnitID, C.UnitTitle " & _
                        "FROM Components C " & _
                        "INNER JOIN DaoGrp DG ON C.ID=DG.ComponentID " & _
                        "INNER JOIN DigitalObjects DO ON DG.DigitalObjectID=DO.DigitalObjectID " & _
                        "WHERE DO.DigitalObjectID IN " & strDigitalObjectIDList

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "Components")

            'myCommand = "SELECT * FROM DaoGrp " & _
            '            "WHERE DigitalObjectID IN " & strDigitalObjectIDList

            myCommand = "SELECT DISTINCT DG.DigitalObjectID, DG.ComponentID " & _
                        "FROM DigitalObjects DO " & _
                        "INNER JOIN DaoGrp DG ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                        "WHERE DO.DigitalObjectID IN " & strDigitalObjectIDList

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DaoGrp")

        Catch e As Exception
            Throw (e)
        Finally
            myReader.Dispose()
        End Try
        Return myData
    End Function





    'Generate preservation information report data
    Public Function GeneratePreservationInfoReportData(ByVal DigitalObjectCollection As DigitalObjectCollection) As PreservationInfoData
        Dim myData As New PreservationInfoData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Dim strDigitalObjectIDList As String = GetDigitalObjectIDList(DigitalObjectCollection)
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = String.Format("SELECT DO.DigitalObjectID, Name, ArchiveDateTime, ArchiveID, " & _
                        "Username, ISNULL(F.QuantityOfTerminalObjects, 0) AS QuantityOfTerminalObjects " & _
                        "FROM DigitalObjects DO " & _
                        "LEFT JOIN " & _
                        "(SELECT DigitalObjectID, COUNT(DigitalObjectID) AS QuantityOfTerminalObjects " & _
                        "FROM Files " & _
                        "GROUP BY DigitalObjectID) AS F " & _
                        "ON DO.DigitalObjectID=F.DigitalObjectID " & _
                        "WHERE DO.DigitalObjectID IN {0};", strDigitalObjectIDList)

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DigitalObjects")

            myCommand = "SELECT PA.*, RM.ReformattingMethod " & _
                        "FROM PreservationAction PA " & _
                        "LEFT JOIN ReformattingMethods RM ON PA.ReformattingMethodID=RM.ReformattingMethodID " & _
                        "WHERE DigitalObjectID IN " & strDigitalObjectIDList

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "PreservationAction")

            myCommand = "SELECT * FROM ReformattingNorms"

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "ReformattingNorms")

            myCommand = "SELECT * FROM PreservationNorms PN " & _
                        "INNER JOIN PreservationAction PA ON PN.PreservationActionID=PA.PreservationActionID " & _
                        "WHERE PA.DigitalObjectID IN " & strDigitalObjectIDList

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "PreservationNorms")

        Catch e As Exception
            Throw (e)
        Finally
            myReader.Dispose()
        End Try
        Return myData
    End Function





    'Generate reformatting methods report data
    Public Function GenerateReformattingMethodsReportData() As ReformattingMethodsData
        Dim myData As New ReformattingMethodsData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = "SELECT * FROM ReformattingMethods ORDER BY ReformattingMethod"

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "ReformattingMethods")

        Catch e As SqlException
            Throw (e)
        Finally
            myReader.Dispose()
        End Try
        Return myData
    End Function


    'Generate reformatting norms report data
    Public Function GenerateReformattingNormsReportData() As ReformattingNormsData
        Dim myData As New ReformattingNormsData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = "SELECT * FROM ReformattingNorms ORDER BY ReformattingNorm"

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "ReformattingNorms")

        Catch e As SqlException
            Throw (e)
        Finally
            myReader.Dispose()
        End Try
        Return myData
    End Function


    'Generate UI digitalizated report data
    Public Function GenerateUIDigitalizationReportData() As UIDigitalizationData
        Dim myData As New UIDigitalizationData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = "SELECT ID, CompleteUnitID, UnitTitle " & _
                        "FROM Components " & _
                        "WHERE Otherlevel='F'"

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "Components")

            myCommand = "SELECT ParentFondsID AS ID, Count(DO.DigitalObjectID) AS UINum, (Count(DO.DigitalObjectID)*100.0)/Count(C.ID) AS UIPercent, Count(C.ID) AS UINumTotal " & _
                        "FROM Components C " & _
                        "LEFT JOIN DaoGrp DG ON C.ID=DG.ComponentID " & _
                        "LEFT JOIN DigitalObjects DO ON DG.DigitalObjectID=DO.DigitalObjectID " & _
                        "WHERE C.OtherLevel='UI' " & _
                        "AND ParentFondsID IN (SELECT ID FROM Components C WHERE Otherlevel='F') " & _
                        "GROUP BY ParentFondsID "

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "NumUI")

        Catch e As Exception
            MsgBox(e.Message)
        End Try
        Return myData
    End Function

#End Region

#Region "Some auxiiary functions"

    Private Function GetProjectIDList(ByVal LVDOCollection As ListView.SelectedListViewItemCollection) As String
        Dim strList As String = "("
        For i As Integer = 0 To LVDOCollection.Count - 1
            strList = strList + CStr(LVDOCollection.Item(i).Tag) + ","
        Next
        strList = strList.TrimEnd(",")
        strList &= ")"
        Return strList
    End Function

    Private Function GetDigitalObjectIDList(ByVal DigitalObjectCollection As DigitalObjectCollection) As String
        Dim strList As String = "("
        For Each DigitalObject As DigitalObject In DigitalObjectCollection
            strList = String.Format("{0}{1},", strList, CStr(DigitalObject.DigitalObjectID))
        Next
        strList = strList.TrimEnd(",")
        strList &= ")"
        Return strList
    End Function

    Private Function GetLogo() As Byte()
        If File.Exists(AppDomain.CurrentDomain.BaseDirectory & LOGO_PATH) Then
            Dim FileStream As FileStream
            Dim BinaryReader As BinaryReader
            FileStream = New FileStream(AppDomain.CurrentDomain.BaseDirectory & LOGO_PATH, FileMode.Open, FileAccess.Read)
            BinaryReader = New BinaryReader(FileStream)
            Dim imgbyte(FileStream.Length) As Byte
            imgbyte = BinaryReader.ReadBytes(Convert.ToInt32(FileStream.Length))
            BinaryReader.Close()
            FileStream.Close()
            Return imgbyte
        End If
    End Function

#End Region

End Class
