Imports System.Data.SqlClient
Imports System.IO
Imports System.Configuration
Imports DigitArq.Utility
Imports DigitArq.Definitions.MPOD

Public Class ReportDataSetBuilder

#Region "Data base connection"

    Private SQLServerSettings As Hashtable = ConfigurationSettings.GetConfig("sqlServerSettings")
    Private ServerAddress As String = SQLServerSettings("ServerAddress")
    Private Database As String = SQLServerSettings("Database")
    Private Username As String = SQLServerSettings("Username")
    Private Password As String = SQLServerSettings("Password")
    Private ConnectTimeout As String = SQLServerSettings("ConnectTimeout")

    Private ConnectionString As String = String.Format("Data Source={0}; Initial Catalog={1}; User ID={2}; password={3}; Connect Timeout={4};", ServerAddress, Database, Username, Password, ConnectTimeout)

#End Region

#Region "UI"

    Public Function GenerateUIDigitalizationReportData(ByVal LVFondsCollection As ListView.SelectedListViewItemCollection) As UIDigitalizationData
        Dim myData As New UIDigitalizationData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Try
            Dim strListFonds As String = GetDigitalObjectIDList(LVFondsCollection)

            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = "SELECT ID, CompleteUnitID, UnitTitle " & _
                        "FROM Components " & _
                        "WHERE Otherlevel='F' " & _
                        "AND ID IN " & strListFonds

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "Components")

            myCommand = "SELECT ParentFondsID AS ID, 1 AS UINum, 1 UIPercent, COUNT(C.OtherLevel) AS UINumTotal " & _
                        "FROM Components C " & _
                        "WHERE C.OtherLevel='UI' AND ParentFondsID IN " & strListFonds & _
                        " GROUP BY ParentFondsID " & _
                        "UNION " & _
                        "SELECT ParentFondsID AS ID, 1 AS UINum, 1 UIPercent, 0 AS UINumTotal " & _
                        "FROM Components C " & _
                        "WHERE ParentFondsID IN " & strListFonds & _
                        " AND ParentFondsID NOT IN " & _
                        "(SELECT ParentFondsID " & _
                        "FROM Components C " & _
                        "WHERE C.OtherLevel='UI' AND ParentFondsID IN " & strListFonds & ")"


            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "NumUI")

            For Each row As DataRow In myData.NumUI
                row.Item("UINum") = CalculateDigitalizatedUI(row.Item("ID"))
                If CInt(row.Item("UINumTotal")) <> 0 Then
                    row.Item("UIPercent") = row.Item("UINum") * 100.0 / row.Item("UINumTotal")
                Else
                    row.Item("UIPercent") = 0
                End If
            Next

        Catch e As Exception
            MsgBox(ErrorMessage("Reports.Error"), MsgBoxStyle.Critical)
        End Try
        Return myData
    End Function

    Private Function CalculateDigitalizatedUI(ByVal ID As Int64) As Integer
        Dim myReader As SqlDataReader
        Dim myConnection As New SqlConnection(ConnectionString)
        Dim myCommand As New SqlCommand
        Try
            myConnection.Open()
            myCommand.Connection = myConnection
            myCommand.CommandText = "SELECT COUNT(DISTINCT ID) " & _
                                    "FROM DigitalObjects DO " & _
                                    "INNER JOIN DaoGrp DG ON DO.DigitalobjectID = DG.DigitalObjectID " & _
                                    "INNER JOIN Components C ON C.ID = DG.ComponentID " & _
                                    "WHERE C.OtherLevel='UI' AND ParentFondsID = " & ID

            Return DBToInt(myCommand.ExecuteScalar)

        Catch e As SqlException
            MsgBox(ErrorMessage("Reports.Error"), MsgBoxStyle.Critical)
        Finally
            myConnection.Close()
        End Try
    End Function

#End Region
    
#Region "Digitalizated documents"

    Public Function GenerateDigitarqDOReportData(ByVal FondID As Int64) As DigitarqODData
        Dim myData As New DigitarqODData
        Dim myReader As SqlDataAdapter
        Dim myCommand As String
        Try
            Dim DataRow As DataRow
            DataRow = myData.Logo.NewRow
            DataRow(0) = GetLogo()
            myData.Logo.Rows.Add(DataRow)

            myCommand = "SELECT DISTINCT ID, ParentID, ParentFondsID, HasChildren, Otherlevel, CompleteUnitID, CountryCode, RepositoryCode, UnitTitle " & _
                        "FROM Components " & _
                        "WHERE ParentFondsID=" & FondID

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "Components")

            myCommand = "SELECT DO.DigitalObjectID, ProjectID, Name, ArchiveID, TopographicalQuota " & _
                        "FROM DigitalObjects DO " & _
                        "INNER JOIN DaoGrp DG ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                        "INNER JOIN Components C ON C.ID=DG.ComponentID " & _
                        "WHERE C.ParentFondsID=" & FondID & _
                        "UNION " & _
                        "SELECT -1, 0, '', '', ''"

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DigitalObjects")


            myCommand = "SELECT DISTINCT C.ID AS ComponentID, ISNULL(DG.DigitalObjectID, -1) AS DigitalObjectID " & _
                        "FROM DaoGrp DG " & _
                        "RIGHT JOIN Components C ON C.ID=DG.ComponentID " & _
                        "WHERE C.ParentFondsID=" & FondID

            myReader = New SqlDataAdapter(myCommand, ConnectionString)
            myReader.Fill(myData, "DaoGrp")

            For Each row As DataRow In myData.Components
                Dim ret(2) As Integer
                Dim value As Int64 = CType(row.Item("ID"), Int64)
                ret = CalculateParentAndChildrenDerivatives(value)
                row.Item("TotalImages") = ret(1)
                row.Item("TotalDerivatives") = ret(2)
            Next

            For Each row As DataRow In myData.DigitalObjects
                row.Item("QuantityOfTerminalObjects") = SelectCountFiles(row.Item("DigitalObjectID"))
            Next

        Catch e As Exception
            MsgBox(ErrorMessage("Reports.Error"), MsgBoxStyle.Critical)
        End Try
        Return myData
    End Function


    Private Function CalculateParentAndChildrenDerivatives(ByVal ID As Integer) As Integer()
        Dim ret(2), ret1(2), ret2(2) As Integer
        Try
            ret1 = CalculateParentDerivativesNumber(ID)
            ret2 = CalculateChildrenDerivativesNumber(ID)

            ret(1) = ret1(1) + ret2(1)
            ret(2) = ret1(2) + ret2(2)
        Catch ex As Exception
            Dim erro As String = ex.ToString
            Dim erro1 As String = ex.ToString
        End Try

        Return ret
    End Function

    Function SelectCountFiles(ByVal DigitalObjectID As Int64) As Integer
        Dim SqlConnection As New SqlConnection(ConnectionString)
        Dim SqlCommand As New SqlCommand
        Try
            SqlConnection.Open()
            SqlCommand.Connection = SqlConnection
            SqlCommand.CommandText = String.Format("SELECT Count(ID) FROM Files " & _
                                    "WHERE DigitalObjectID = {0};", DigitalObjectID)
            Return SqlCommand.ExecuteScalar
        Catch ex As Exception
            Throw (ex)
        Finally
            SqlCommand.Dispose()
            SqlConnection.Close()
            SqlConnection.Dispose()
        End Try
    End Function

    Private Function CalculateParentDerivativesNumber(ByVal ID As Int64) As Integer()
        Dim myReader As SqlDataReader
        Dim myConnection As New SqlConnection(ConnectionString)
        Dim myCommand As New SqlCommand
        Dim value As Integer
        Dim ret(2) As Integer
        Try
            myConnection.Open()
            myCommand.Connection = myConnection
            'myCommand.CommandText = "SELECT SUM(QuantityOfTerminalObjects) " & _
            '                        "FROM DigitalObjects DO " & _
            '                        "INNER JOIN DaoGrp DG ON DO.DigitalObjectID=DG.DigitalObjectID " & _
            '                        "WHERE DG.ComponentID=" & ID & " AND Type = 0"

            myCommand.CommandText = String.Format("SELECT COUNT(F.ID) " & _
                                    "FROM Files F " & _
                                    "INNER JOIN DigitalObjects DO ON F.DigitalObjectID=DO.DigitalObjectID " & _
                                    "INNER JOIN DaoGrp DG ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                                    "WHERE DG.ComponentID={0} AND Type={1};", ID, 0)
            value = DBToInt(myCommand.ExecuteScalar)

            myCommand.CommandText = String.Format("SELECT Count(FileID) " & _
                                    "FROM DaoGrp " & _
                                    "WHERE ComponentID={0} AND Type={1};", ID, 1)
            ret(1) = value + DBToInt(myCommand.ExecuteScalar)

            myCommand.CommandText = String.Format("SELECT COUNT(F.ID) " & _
                                    "FROM Files F " & _
                                    "INNER JOIN DigitalObjects DO ON F.DigitalObjectID=DO.DigitalObjectID " & _
                                    "INNER JOIN DaoGrp DG ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                                    "WHERE F.HasDissemination={0} AND DG.ComponentID={1} AND Type={2} " & _
                                    "GROUP BY DO.DigitalObjectID;", 1, ID, 0)
            value = DBToInt(myCommand.ExecuteScalar)

            myCommand.CommandText = String.Format("SELECT COUNT(ID) " & _
                                    "FROM Files " & _
                                    "WHERE HasDissemination={0} " & _
                                    "AND ID IN (SELECT FileID FROM DaoGrp WHERE Type={1} AND ComponentID={2});", 1, 1, ID)
            ret(2) = value + DBToInt(myCommand.ExecuteScalar)

            Return ret
        Catch e As SqlException
            MsgBox(e)
            Throw (e)
        Finally
            myConnection.Close()
        End Try
    End Function

    Private Function CalculateChildrenDerivativesNumber(ByVal ID As Integer, Optional ByVal images As Integer = 0, Optional ByVal derivatives As Integer = 0) As Integer()
        Dim myReader As SqlDataReader
        Dim myConnection As New SqlConnection(ConnectionString)
        Dim myCommand As New SqlCommand
        Dim value As Integer
        Dim ret(2) As Integer
        Try
            myConnection.Open()
            myCommand.Connection = myConnection
            'myCommand.CommandText = "SELECT SUM(QuantityOfTerminalObjects) " & _
            '                        "FROM DigitalObjects DO " & _
            '                        "INNER JOIN DaoGrp DG ON DO.DigitalObjectID=DG.DigitalObjectID " & _
            '                        "INNER JOIN Components C ON C.ID=DG.ComponentID " & _
            '                        "WHERE C.ParentID=" & ID & " AND DG.Type = 0 " & _
            '                        "GROUP BY C.ParentID"

            myCommand.CommandText = String.Format("SELECT COUNT(F.ID) " & _
                                   "FROM Files F " & _
                                   "INNER JOIN DigitalObjects DO ON F.DigitalObjectID=DO.DigitalObjectID " & _
                                   "INNER JOIN DaoGrp DG ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                                   "INNER JOIN Components C ON C.ID=DG.ComponentID " & _
                                   "WHERE C.ParentID={0} AND Type={1};", ID, 0)
            value = images + DBToInt(myCommand.ExecuteScalar())

            myCommand.CommandText = String.Format("SELECT COUNT(DG.FileID) " & _
                                    "FROM DaoGrp DG " & _
                                    "INNER JOIN Components C ON C.ID=DG.ComponentID " & _
                                    "WHERE C.ParentID={0} AND DG.Type={1};", ID, 1)
            ret(1) = value + DBToInt(myCommand.ExecuteScalar())

            myCommand.CommandText = String.Format("SELECT COUNT(F.ID) " & _
                                    "FROM Files F " & _
                                    "INNER JOIN DigitalObjects DO ON F.DigitalObjectID=DO.DigitalObjectID " & _
                                    "INNER JOIN DaoGrp DG ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                                    "INNER JOIN Components C ON C.ID=DG.ComponentID " & _
                                    "WHERE F.HasDissemination={0} AND C.ParentID={1} AND DG.Type={2};", 1, ID, 0)

            value = derivatives + DBToInt(myCommand.ExecuteScalar())

            myCommand.CommandText = String.Format("SELECT COUNT(F.ID) " & _
                                    "FROM Files F " & _
                                    "WHERE F.HasDissemination={0} " & _
                                    "AND F.ID IN " & _
                                    "(SELECT FileID FROM DaoGrp DG " & _
                                    "INNER JOIN Components C ON C.ID=DG.ComponentID " & _
                                    "WHERE C.ParentID={1} AND DG.Type={2});", 1, ID, 1)

            ret(2) = value + DBToInt(myCommand.ExecuteScalar)

            myCommand.CommandText = String.Format("SELECT ID, HasChildren " & _
                                    "FROM Components " & _
                                    "WHERE ParentID={0};", ID)
            myReader = myCommand.ExecuteReader
            While myReader.Read
                If DBToBool(myReader.Item("HasChildren")) Then
                    ret = CalculateChildrenDerivativesNumber(myReader.Item("ID"), ret(1), ret(2))
                End If
            End While
            Return ret
        Catch e As SqlException
            MsgBox(e.ToString)
            Throw (e)
        Finally
            myConnection.Close()
        End Try
    End Function

#End Region

#Region "Some auxiliary functions"

    Private Function GetDigitalObjectIDList(ByVal LVDOCollection As ListView.SelectedListViewItemCollection) As String
        Dim strList As String = "("
        For i As Integer = 0 To LVDOCollection.Count - 1
            strList = strList + CStr(LVDOCollection.Item(i).Tag) + ","
        Next
        strList = strList.TrimEnd(",")
        strList = strList + ")"
        Return strList
    End Function

    Private Function GetLogo() As Byte()
        If File.Exists(AppDomain.CurrentDomain.BaseDirectory & LOGO_PATH) Then
            Dim FileStream As FileStream
            Dim BinaryReader As BinaryReader
            FileStream = New FileStream(AppDomain.CurrentDomain.BaseDirectory & LOGO_PATH, FileMode.Open, FileAccess.Read)
            BinaryReader = New BinaryReader(FileStream)
            Dim imgbyte(FileStream.Length) As Byte
            imgbyte = BinaryReader.ReadBytes(Convert.ToInt32(FileStream.Length))
            BinaryReader.Close()
            FileStream.Close()
            Return imgbyte
        End If
    End Function

#End Region

End Class
