Imports System.Data.SqlClient
Imports System.Configuration
Imports DigitArq.Data.Vault.Client
Imports DigitArq.Definitions.MDNP
Imports DigitArq.Utility

Namespace MDNP

    Public Class DOImage
        Inherits DOFile

#Region "Internal Properties (private)"
        '*************************************************************************
        Private myServerAddress As String
        Private myDatabase As String
        Private myUsername As String
        Private myPassword As String
        Private myConnectionString As String

        Private myID As Int64
        Private myProperties As Hashtable
        Private myResolution As String
        Private myWidth As String
        Private myHeight As String
        Private myLength As String
        Private myColorSpace As String
        Private myBitsPerPixel As String
        Private myImageType As ImageType
        Private myCapacity As Capacities

        Public Enum Capacities
            Image
            Description
            Tree
        End Enum

        Public Enum ImageType
            Dissemination
            Thumbnail
            Original
        End Enum

#End Region

#Region "Constructors"

        Public Sub New(ByVal ID As Int64, ByVal Capacity As Capacities, Optional ByVal Type As ImageType = ImageType.Dissemination)
            Dim SqlServerSettings As Hashtable = ConfigurationSettings.GetConfig("sqlServerSettings")

            myServerAddress = SqlServerSettings("ServerAddress")
            myDatabase = SqlServerSettings("Database")
            myUsername = SqlServerSettings("Username")
            myPassword = SqlServerSettings("Password")

            myConnectionString = String.Format("Data Source={0}; Initial Catalog={1}; User ID={2}; password={3}", myServerAddress, myDatabase, myUsername, myPassword)

            myID = ID
            myImageType = Type
            myCapacity = Capacity
            Download()
        End Sub

#End Region

#Region "Public Properties"

        Property Resolution() As String
            Get
                Return myResolution
            End Get
            Set(ByVal Value As String)
                myResolution = Value
            End Set
        End Property

        Property Width() As String
            Get
                Return myWidth
            End Get
            Set(ByVal Value As String)
                myWidth = Value
            End Set
        End Property

        Property Height() As String
            Get
                Return myHeight
            End Get
            Set(ByVal Value As String)
                myHeight = Value
            End Set
        End Property

        Property ColorSpace() As String
            Get
                Return myColorSpace
            End Get
            Set(ByVal Value As String)
                myColorSpace = Value
            End Set
        End Property

        Property BitsPerPixel() As String
            Get
                Return myBitsPerPixel
            End Get
            Set(ByVal Value As String)
                myBitsPerPixel = Value
            End Set
        End Property

        Property Properties() As Hashtable
            Get
                Return myProperties
            End Get
            Set(ByVal Value As Hashtable)
                myProperties = Value
            End Set
        End Property

#End Region

#Region "Methods"

        Public Sub Download()
            Select Case myCapacity
                Case Capacities.Description : DownloadMetadata()
                Case Capacities.Image : DownloadImage()
            End Select
        End Sub

        Private Sub DownloadMetadata()
            Dim hash As New Hashtable
            Dim SqlConnection As New SqlConnection(myConnectionString)
            Dim SqlDataReader As SqlDataReader
            Dim SqlCommand As New SqlCommand
            Try
                SqlConnection.Open()
                SqlCommand.Connection = SqlConnection
                SqlCommand.CommandText = String.Format("SELECT * FROM Files WHERE ID = '{0}'", myID)
                SqlDataReader = SqlCommand.ExecuteReader()
                While SqlDataReader.Read()
                    Me.ID = DBToInt(SqlDataReader.Item("ID"))
                    Me.Name = DBToStr(SqlDataReader.Item("Name"))
                    Me.Format = DBToStr(SqlDataReader.Item("Format"))
                    Me.Length = DBToStr(SqlDataReader.Item("Length"))
                    Me.OriginalURL = DBToStr(SqlDataReader.Item("OriginalURL"))
                    Me.DisseminationURL = DBToStr(SqlDataReader.Item("DisseminationURL"))
                    Me.ThumbnailURL = DBToStr(SqlDataReader.Item("ThumbnailURL"))
                End While
                SqlDataReader.Close()

                Me.Properties = SelectProperties(myID)

                Me.Resolution = Me.Properties.Item("Resoluo (dpi)")
                Me.Width = Me.Properties.Item("Largura")
                Me.Height = Me.Properties.Item("Altura")
                Me.ColorSpace = Me.Properties.Item("Esquema cor")
                Me.BitsPerPixel = Me.Properties.Item("Profundidade bits")
            Catch ex As Exception

            Finally
                SqlCommand.Dispose()
                SqlDataReader.Close()
                SqlConnection.Close()
                SqlConnection.Dispose()
            End Try
        End Sub

        Public Function SelectProperties(ByVal FileID As Int64) As Hashtable
            Dim hash As New Hashtable
            Dim SqlConnection As New SqlConnection(myConnectionString)
            Dim SqlDataReader As SqlDataReader
            Dim SqlCommand As New SqlCommand
            Try
                SqlConnection.Open()
                SqlCommand.Connection = SqlConnection
                SqlCommand.CommandText = String.Format("SELECT Property, Value FROM Properties " & _
                                        "WHERE FileID={0}", FileID)
                SqlDataReader = SqlCommand.ExecuteReader
                While SqlDataReader.Read
                    hash.Add(SqlDataReader.Item("Property"), SqlDataReader.Item("Value"))
                End While
            Catch e As SqlException
                'MsgBoxSqlException(e)
            Finally
                SqlCommand.Dispose()
                SqlDataReader.Close()
                SqlConnection.Close()
                SqlConnection.Dispose()
            End Try
            Return hash
        End Function

        Private Sub DownloadImage()
            Dim SQLConnection As SqlConnection = New SqlConnection(myConnectionString)
            Dim SQLCommand As New SqlCommand
            SQLCommand.Connection = SQLConnection
            Try
                SQLConnection.Open()
                Dim DataReader As SqlDataReader

                SQLCommand.CommandText = String.Format("SELECT * FROM Files " & _
                                         "WHERE ID = '{0}'", myID)
                DataReader = SQLCommand.ExecuteReader()
                While DataReader.Read()
                    Me.ID = DBToInt(DataReader.Item("ID"))
                    Me.Name = DBToStr(DataReader.Item("Name"))
                    Me.Format = DBToStr(DataReader.Item("Format"))
                    Me.Length = DBToStr(DataReader.Item("Length"))
                    Me.OriginalURL = DBToStr(DataReader.Item("OriginalURL"))
                    Me.DisseminationURL = DBToStr(DataReader.Item("DisseminationURL"))
                    Me.ThumbnailURL = DBToStr(DataReader.Item("ThumbnailURL"))
                End While
                DataReader.Close()

                Select Case myImageType
                    Case ImageType.Original
                        If OriginalURL.StartsWith(SQLSERVER_STORAGE) Then
                            Dim BLOBID As Int64 = Me.OriginalURL.Substring(SQLSERVER_STORAGE.Length)
                            SQLCommand.CommandText = String.Format("SELECT Data FROM BLOBS " & _
                                                     "WHERE ID = '{0}'", BLOBID)
                            DataReader = SQLCommand.ExecuteReader()
                            While DataReader.Read()
                                Me.Data = DBToByte(DataReader.Item("Data"))
                            End While
                        ElseIf OriginalURL.StartsWith(OBJECTVAULT_STORAGE) Then
                            Dim ID As String = Me.OriginalURL.Substring(OBJECTVAULT_STORAGE.Length)
                            Dim vault As New ObjectVaultClient(OBJECT_VAULT_SERVER_ADDRESS)
                            Me.Data = vault.Retrieve(ID)
                        End If

                    Case ImageType.Dissemination
                        If DisseminationURL.StartsWith(SQLSERVER_STORAGE) Then
                            Dim BLOBID As Int64 = Me.DisseminationURL.Substring(SQLSERVER_STORAGE.Length)
                            SQLCommand.CommandText = String.Format("SELECT Data FROM BLOBS " & _
                                                     "WHERE ID = '{0}'", BLOBID)
                            DataReader = SQLCommand.ExecuteReader()
                            While DataReader.Read()
                                Me.Data = DBToByte(DataReader.Item("Data"))
                            End While
                        ElseIf DisseminationURL.StartsWith(OBJECTVAULT_STORAGE) Then
                            Dim ID As String = Me.DisseminationURL.Substring(OBJECTVAULT_STORAGE.Length)
                            Dim vault As New ObjectVaultClient(OBJECT_VAULT_SERVER_ADDRESS)
                            Me.Data = vault.Retrieve(ID)
                        End If

                    Case ImageType.Thumbnail
                        If ThumbnailURL.StartsWith(SQLSERVER_STORAGE) Then
                            Dim BLOBID As Int64 = Me.ThumbnailURL.Substring(SQLSERVER_STORAGE.Length)
                            SQLCommand.CommandText = String.Format("SELECT Data FROM BLOBS " & _
                                                     "WHERE ID = '{0}'", BLOBID)
                            DataReader = SQLCommand.ExecuteReader()
                            While DataReader.Read()
                                Me.Data = DBToByte(DataReader.Item("Data"))
                            End While

                        ElseIf ThumbnailURL.StartsWith(OBJECTVAULT_STORAGE) Then
                            Dim ID As String = Me.ThumbnailURL.Substring(OBJECTVAULT_STORAGE.Length)
                            Dim vault As New ObjectVaultClient(OBJECT_VAULT_SERVER_ADDRESS)
                            Me.Data = vault.Retrieve(ID)

                        End If

                End Select

                DataReader.Close()
            Catch ex As SqlException
            Catch ex As Exception
                Dim erro As String = ex.ToString
                Dim erro1 As String = ex.ToString
            Finally
                SQLConnection.Close()
            End Try
        End Sub

#End Region

    End Class

End Namespace

Namespace MDNP

    Public Class DOImageCollection
        Inherits CollectionBase

        Public Function Add(ByVal ID As Int64, ByVal DigitalObjectID As Int64, _
                            ByVal Name As String) As Integer
            Return List.Add(New DOImage(ID, DigitalObjectID, Name))
        End Function

        Public Sub Remove(ByVal Item As DOImage)
            List.Remove(Item)
        End Sub

        Public Function Item(ByVal index As Integer) As DOFile
            Return List.Item(index)
        End Function

    End Class

End Namespace

