Imports System.Data.SqlClient
Imports DigitArq.Utility
Imports System.Configuration

Namespace MDNP

    Public Class SQLLazyNode
        Inherits LazyNode

#Region "Internal Properties (private)"

        Private myServerAddress As String
        Private myDatabase As String
        Private myUsername As String
        Private myPassword As String
        Private myConnectTimeout As String
        Private myConnectionString As String

        Private myID As Integer
        Private myParentID As Integer
        Private myCapacity As Integer

        Public Enum Capacities
            Full
            Medium
            Minimum
        End Enum

#End Region

#Region "Initializations"

        Public Sub New(ByVal ServerAddress As String, ByVal Database As String, ByVal Username As String, ByVal Password As String, ByVal ID As Integer, ByVal Capacity As Capacities)

            myConnectionString = String.Format("Data Source={0}; Initial Catalog={1}; User ID={2}; Password={3}", ServerAddress, Database, Username, Password)
            myID = ID
            myCapacity = Capacity
            Download()
        End Sub

        Public Sub New(ByVal ID As Integer, ByVal Capacity As Capacities)
            Dim hshDatabase As Hashtable = ConfigurationSettings.GetConfig("sqlServerSettings")

            myServerAddress = hshDatabase("ServerAddress")
            myDatabase = hshDatabase("Database")
            myUsername = hshDatabase("Username")
            myPassword = hshDatabase("Password")
            myConnectTimeout = hshDatabase("ConnectTimeout")

            myConnectionString = String.Format("Data Source={0}; Initial Catalog={1}; User ID={2}; password={3}", myServerAddress, myDatabase, myUsername, myPassword)
            myID = ID
            myCapacity = Capacity
            Download()
        End Sub

        Public Sub New()
            Dim hshDatabase As Hashtable = ConfigurationSettings.GetConfig("sqlServerSettings")

            myServerAddress = hshDatabase("ServerAddress")
            myDatabase = hshDatabase("Database")
            myUsername = hshDatabase("Username")
            myPassword = hshDatabase("Password")
            myConnectTimeout = hshDatabase("ConnectTimeout")

            myConnectionString = String.Format("Data Source={0}; Initial Catalog={1}; User ID={2}; password={3}", myServerAddress, myDatabase, myUsername, myPassword)
        End Sub

#End Region

#Region "Specific methods"

        Public Overrides ReadOnly Property ID() As Object
            Get
                Return myID
            End Get
        End Property

        Public Function LoadControlAccessTypes() As DataSet
            Dim SQLConnection As SQLConnection = New SQLConnection(myConnectionString)
            Dim SqlDataAdapter As SqlDataAdapter
            Dim DataSet As DataSet
            Try
                SQLConnection.Open()
                Dim SelectCommandText As String = "SELECT DISTINCT CAT.ID, CAT.Type " & _
                                                  "FROM ControlAccess CA " & _
                                                  "INNER JOIN ControlAccessItems CAI ON CA.ItemID=CAI.ID " & _
                                                  "INNER JOIN ControlAccessTypes CAT ON CAI.TypeID=CAT.ID"

                SqlDataAdapter = New SqlDataAdapter(SelectCommandText, SQLConnection)
                DataSet = New DataSet
                SqlDataAdapter.Fill(DataSet)

                Return DataSet
            Catch ex As Exception
                Dim erro As String = ex.ToString
                Dim aa As String = "aa"
            Finally
                SqlDataAdapter = Nothing
                SQLConnection.Close()
                SQLConnection.Dispose()
            End Try
        End Function


        Public Function LoadControlAccessItems(ByVal TypeID As Integer) As DataSet
            Dim SQLConnection As SQLConnection = New SQLConnection(myConnectionString)
            Dim SqlDataAdapter As SqlDataAdapter
            Dim DataSet As DataSet
            Try
                SQLConnection.Open()
                Dim SelectCommandtext As String = String.Format("SELECT DISTINCT ControlAccessItems.ID, ControlAccessItems.TypeID, ControlAccessItems.Item " & _
                                                                "FROM ControlAccessItems, ControlAccess WHERE TypeID='{0}' AND ControlAccess.ItemID=ControlAccessItems.ID", TypeID)

                SqlDataAdapter = New SqlDataAdapter(SelectCommandtext, SQLConnection)
                DataSet = New DataSet
                SqlDataAdapter.Fill(DataSet)

                Return DataSet
            Catch ex As Exception

            Finally
                SqlDataAdapter = Nothing
                SQLConnection.Close()
            End Try
        End Function

#End Region

#Region "LazyNode Methods"

        Public Overrides Sub Download()
            Select Case myCapacity
                Case Capacities.Full : DownloadFull()
                Case Capacities.Medium : DownloadMedium()
                Case Capacities.Minimum : DownloadMinimum()
            End Select
        End Sub

        Private Sub DownloadMinimum()
            Dim SQLConnection As SQLConnection = New SQLConnection(myConnectionString)
            Dim SQLCommand As New SQLCommand
            Dim DataReader As SqlDataReader
            Dim Transaction As SqlTransaction
            SQLCommand.Connection = SQLConnection
            Try
                SQLConnection.Open()
                Transaction = SQLConnection.BeginTransaction(IsolationLevel.ReadCommitted)
                SQLCommand.Transaction = Transaction
                SQLCommand.CommandText = String.Format("SELECT ID, Visible, Valid, CompleteUnitID, UnitID, ParentID, " & _
                                                       "OtherLevel, UnitTitle, UnitTitleType, " & _
                                                       "UnitDateInitial, UnitDateFinal, UnitDateFinalCertainty, " & _
                                                       "UnitDateInitialCertainty " & _
                                                       "FROM Components WHERE ID = '{0}'", myID)

                DataReader = SQLCommand.ExecuteReader()
                While DataReader.Read()
                    CompleteUnitId = DBToStr(DataReader.Item("CompleteUnitId"))
                    UnitId = DBToStr(DataReader.Item("UnitId"))
                    ParentID = DBToInt(DataReader.Item("ParentID"))
                    OtherLevel = DBToStr(DataReader.Item("OtherLevel"))
                    UnitTitle = DBToStr(DataReader.Item("UnitTitle"))
                    UnitTitleType = DBToStr(DataReader.Item("UnitTitleType"))
                    UnitDateInitial = DBToStr(DataReader.Item("UnitDateInitial"))
                    UnitDateFinal = DBToStr(DataReader.Item("UnitDateFinal"))
                    UnitDateFinalCertainty = DBToBool(DataReader.Item("UnitDateFinalCertainty"))
                    UnitDateInitialCertainty = DBToBool(DataReader.Item("UnitDateInitialCertainty"))
                    Visible = DBToBool(DataReader.Item("Visible"))
                    Valid = DBToBool(DataReader.Item("Valid"))
                End While
                Transaction.Commit()
            Catch ex As SqlException
                Transaction.Rollback()
            Catch ex As Exception

            Finally
                DataReader.Close()
                SQLCommand.Dispose()
                SQLConnection.Close()
                SQLConnection.Dispose()
            End Try
        End Sub

        Private Sub DownloadMedium()
            Dim SQLConnection As SQLConnection = New SQLConnection(myConnectionString)
            Dim SQLCommand As New SQLCommand
            Dim DataReader As SqlDataReader
            SQLCommand.Connection = SQLConnection

            Dim Transaction As SqlTransaction
            Try
                SQLConnection.Open()
                Transaction = SQLConnection.BeginTransaction(IsolationLevel.ReadCommitted)
                SQLCommand.Transaction = Transaction
                SQLCommand.CommandText = String.Format("SELECT ID, Visible, Valid, CompleteUnitID, UnitTitle, UnitTitleType, " & _
                                                       "UnitDateInitialCertainty, UnitDateInitial, UnitDateFinalCertainty, " & _
                                                       "UnitDateFinal, OtherLevel, PhysLoc " & _
                                                       "FROM Components WHERE ID = '{0}'", myID)
                DataReader = SQLCommand.ExecuteReader()
                While DataReader.Read()
                    CompleteUnitId = DBToStr(DataReader.Item("CompleteUnitID")).ToUpper
                    UnitTitle = DBToStr(DataReader.Item("UnitTitle"))
                    UnitTitleType = DBToStr(DataReader.Item("UnitTitleType"))
                    OtherLevel = DBToStr(DataReader.Item("OtherLevel"))
                    PhysLoc = DBToStr(DataReader.Item("PhysLoc"))
                    UnitDateFinalCertainty = DBToBool(DataReader.Item("UnitDateFinalCertainty"))
                    UnitDateInitialCertainty = DBToBool(DataReader.Item("UnitDateInitialCertainty"))
                    UnitDateInitial = DBToStr(DataReader.Item("UnitDateInitial"))
                    UnitDateFinal = DBToStr(DataReader.Item("UnitDateFinal"))
                    Visible = DBToBool(DataReader.Item("Visible"))
                    Valid = DBToBool(DataReader.Item("Valid"))
                End While
                DataReader.Close()

                SQLCommand.CommandText = String.Format("SELECT COUNT(FondsID) FROM EAC " & _
                                                       "WHERE FondsID = '{0}'", myID)
                Eac = CType(SQLCommand.ExecuteScalar, Integer) = 1

                SQLCommand.CommandText = String.Format("SELECT DISTINCT DG.ComponentID, DG.DigitalObjectID, DG.FileID, DG.Type, " & _
                                            "DO.Name AS DOName, DO.ArchiveID, DO.TopographicalQuota, DO.Active, DOF.Name AS DOFileName " & _
                                            "FROM DaoGrp DG " & _
                                            "INNER JOIN DigitalObjects DO ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                                            "INNER JOIN Files DOF ON DOF.ID=DG.FileID " & _
                                            "WHERE DG.ComponentID = {0} AND DG.Type = 1 " & _
                                            "UNION " & _
                                            "SELECT DISTINCT DG.ComponentID, DG.DigitalObjectID, DG.FileID, DG.Type, " & _
                                            "DO.Name AS DOName, DO.ArchiveID, DO.TopographicalQuota, DO.Active, '' " & _
                                            "FROM DaoGrp DG " & _
                                            "INNER JOIN DigitalObjects DO ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                                            "WHERE DG.ComponentID={0} AND DG.Type= 0 ", ID)
                DataReader = SQLCommand.ExecuteReader()
                DigitalObjectExists = DataReader.HasRows()

                If Not DigitalObjectExists Then
                    DigitalObjectID = CType(-1, Int64)
                End If

                While DataReader.Read()
                    If DataReader.Item("Active") Then
                        Dim DaoGrpItem As New DaoGrp
                        DaoGrpItem.ComponentID = DataReader.Item("ComponentID")
                        DaoGrpItem.DigitalObjectID = DataReader.Item("DigitalObjectID")
                        DaoGrpItem.DigitalObjectName = DataReader.Item("DOName")
                        DaoGrpItem.FileID = DataReader.Item("FileID")
                        DaoGrpItem.FileName = DataReader.Item("DOFileName")
                        DaoGrpItem.Type = DataReader.Item("Type")
                        DaoGrpItem.ArchiveID = DataReader.Item("ArchiveID")
                        DaoGrpItem.TopographicalQuota = DataReader.Item("TopographicalQuota")

                        If CInt(DataReader.Item("Type")) = 0 Then
                            DaoGrpItem.QuantityOfFiles = GetQuantityOfFiles(DataReader.Item("DigitalObjectID"))
                        Else
                            DaoGrpItem.QuantityOfFiles = 1
                        End If
                        DaoGrp.Add(DaoGrpItem)
                    End If
                End While
                DataReader.Close()

                If (OtherLevel = "D" Or OtherLevel = "DC" Or OtherLevel = "UI") Then
                    IsDSDCUI = True
                Else
                    IsDSDCUI = False
                End If

                Transaction.Commit()
            Catch ex As SqlException
                Transaction.Rollback()
            Catch ex As Exception

            Finally
                DataReader.Close()
                SQLCommand.Dispose()
                SQLConnection.Close()
                SQLConnection.Dispose()
            End Try
        End Sub

        Private Function GetQuantityOfFiles(ByVal DigitalObjectID As Int64) As Integer
            Dim SQLConnection As SQLConnection = New SQLConnection(myConnectionString)
            Dim SQLCommand As New SQLCommand
            SQLCommand.Connection = SQLConnection
            Try
                SQLConnection.Open()
                SQLCommand.CommandText = String.Format("SELECT COUNT(ID) FROM Files " & _
                                                       "WHERE DigitalObjectID='{0}'", DigitalObjectID)

                Return SQLCommand.ExecuteScalar
            Catch ex As Exception
                DigitalObjectExists = False
            Finally
                SQLCommand.Dispose()
                SQLConnection.Close()
                SQLConnection.Dispose()
            End Try
        End Function


        Private Sub DownloadFull()
            Dim SQLConnection As SQLConnection = New SQLConnection(myConnectionString)
            Dim SQLCommand As New SQLCommand
            SQLCommand.Connection = SQLConnection
            Dim DataReader As SqlDataReader
            Dim Transaction As SqlTransaction
            Try
                SQLConnection.Open()
                Transaction = SQLConnection.BeginTransaction(IsolationLevel.ReadCommitted)
                SQLCommand.Transaction = Transaction
                SQLCommand.CommandText = String.Format("SELECT * FROM Components WHERE ID = '{0}'", myID)

                DataReader = SQLCommand.ExecuteReader()
                While DataReader.Read()
                    ParentID = DBToInt(DataReader.Item("ParentID"))
                    UnitId = DBToStr(DataReader.Item("UnitID"))
                    CompleteUnitId = DBToStr(DataReader.Item("CompleteUnitID")).ToUpper
                    UnitTitle = DBToStr(DataReader.Item("UnitTitle"))
                    UnitTitleType = DBToStr(DataReader.Item("UnitTitleType"))
                    Visible = DBToBool(DataReader.Item("Visible"))
                    Valid = DBToBool(DataReader.Item("Valid"))
                    OtherLevel = DBToStr(DataReader.Item("OtherLevel"))
                    CountryCode = DBToStr(DataReader.Item("CountryCode"))
                    RepositoryCode = DBToStr(DataReader.Item("RepositoryCode"))
                    Abstract = DBToStr(DataReader.Item("Abstract"))
                    AccessRestrict = DBToStr(DataReader.Item("AccessRestrict"))
                    Accruals = DBToStr(DataReader.Item("Accruals"))
                    AcqInfo = DBToStr(DataReader.Item("AcqInfo"))
                    AltFormAvail = DBToStr(DataReader.Item("AltFormAvail"))
                    Appraisal = DBToStr(DataReader.Item("Appraisal"))
                    Arrangement = DBToStr(DataReader.Item("Arrangement"))
                    BiogHist = DBToStr(DataReader.Item("BiogHist"))
                    CustodHist = DBToStr(DataReader.Item("CustodHist"))

                    Dimensions = DBToStr(DataReader.Item("Dimensions"))
                    ExtentBook = DBToInt(DataReader.Item("ExtentBook"))
                    ExtentBox = DBToInt(DataReader.Item("ExtentBox"))
                    ExtentCapilha = DBToInt(DataReader.Item("ExtentCapilha"))
                    ExtentFolder = DBToInt(DataReader.Item("ExtentFolder"))
                    ExtentMacete = DBToInt(DataReader.Item("ExtentMacete"))
                    ExtentMaco = DBToInt(DataReader.Item("ExtentMaco"))
                    ExtentMl = DBToInt(DataReader.Item("ExtentMl"))
                    ExtentRoll = DBToInt(DataReader.Item("ExtentRoll"))
                    ExtentEnvelope = DBToInt(DataReader.Item("ExtentEnvelope"))
                    ExtentAlbum = DBToInt(DataReader.Item("ExtentAlbum"))

                    GenreForm = DBToStr(DataReader.Item("GenreForm"))
                    GeogName = DBToStr(DataReader.Item("GeogName"))
                    LangMaterial = DBToStr(DataReader.Item("LangMaterial"))
                    LegalStatus = DBToStr(DataReader.Item("LegalStatus"))
                    MaterialSpec = DBToStr(DataReader.Item("MaterialSpec"))
                    Note = DBToStr(DataReader.Item("Note"))
                    OriginalNumbering = DBToStr(DataReader.Item("OriginalNumbering"))
                    OriginalsLoc = DBToStr(DataReader.Item("OriginalsLoc"))
                    OriginationAuthor = DBToStr(DataReader.Item("OriginationAuthor"))
                    OriginationAuthorAddress = DBToStr(DataReader.Item("OriginationAuthorAddress"))
                    OriginationDestination = DBToStr(DataReader.Item("OriginationDestination"))
                    OriginationDestinationAddress = DBToStr(DataReader.Item("OriginationDestinationAddress"))
                    OriginationNotary = DBToStr(DataReader.Item("OriginationNotary"))
                    OriginationNotary = DBToStr(DataReader.Item("OriginationNotary"))
                    OriginationScrivener = DBToStr(DataReader.Item("OriginationScrivener"))
                    OtherFindAid = DBToStr(DataReader.Item("OtherFindAid"))
                    PhysFacet = DBToStr(DataReader.Item("PhysFacet"))
                    PhysLoc = DBToStr(DataReader.Item("PhysLoc"))
                    PreviousLoc = DBToStr(DataReader.Item("PreviousLoc"))
                    PhysTech = DBToStr(DataReader.Item("PhysTech"))
                    PreferCite = DBToStr(DataReader.Item("PreferCite"))
                    ProcessInfoDate = DBToStr(DataReader.Item("ProcessInfoDate"))
                    ProcessInfoName = DBToStr(DataReader.Item("ProcessInfoName"))
                    RelatedMaterial = DBToStr(DataReader.Item("RelatedMaterial"))
                    Repository = DBToStr(DataReader.Item("Repository"))
                    ScopeContent = DBToStr(DataReader.Item("ScopeContent"))
                    SeparatedMaterial = DBToStr(DataReader.Item("SeparatedMaterial"))
                    UnitDateInitial = DBToStr(DataReader.Item("UnitDateInitial"))
                    UnitDateFinal = DBToStr(DataReader.Item("UnitDateFinal"))
                    UnitDateFinalCertainty = DBToBool(DataReader.Item("UnitDateFinalCertainty"))
                    UnitDateInitialCertainty = DBToBool(DataReader.Item("UnitDateInitialCertainty"))
                    UseRestrict = DBToStr(DataReader.Item("UseRestrict"))
                    FilePlan = DBToStr(DataReader.Item("FilePlan"))
                    UnitTitleType = DBToStr(DataReader.Item("UnitTitleType"))
                    ExtentPage = DBToInt(DataReader.Item("ExtentPage"))
                    ExtentLeaf = DBToInt(DataReader.Item("ExtentLeaf"))
                    ExtentOther = DBToInt(DataReader.Item("ExtentOther"))
                    UnitDateBulk = DBToStr(DataReader.Item("UnitDateBulk"))
                    ContainerType = DBToStr(DataReader.Item("ContainerType"))
                End While
                DataReader.Close()

                SQLCommand.CommandText = String.Format("SELECT COUNT(FondsID) FROM EAC " & _
                                                       "WHERE FondsID = '{0}'", myID)
                Eac = CType(SQLCommand.ExecuteScalar, Integer) = 1

                SQLCommand.CommandText = String.Format("SELECT DISTINCT DG.ComponentID, DG.DigitalObjectID, DG.FileID, DG.Type, " & _
                                            "DO.Name AS DOName, DO.ArchiveID, DO.TopographicalQuota, DO.Active, DOF.Name AS DOFileName " & _
                                            "FROM DaoGrp DG " & _
                                            "INNER JOIN DigitalObjects DO ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                                            "INNER JOIN Files DOF ON DOF.ID=DG.FileID " & _
                                            "WHERE DG.ComponentID = {0} AND DG.Type = 1 " & _
                                            "UNION " & _
                                            "SELECT DISTINCT DG.ComponentID, DG.DigitalObjectID, DG.FileID, DG.Type, " & _
                                            "DO.Name AS DOName, DO.ArchiveID, DO.TopographicalQuota, DO.Active, '' " & _
                                            "FROM DaoGrp DG " & _
                                            "INNER JOIN DigitalObjects DO ON DO.DigitalObjectID=DG.DigitalObjectID " & _
                                            "WHERE DG.ComponentID={0} AND DG.Type= 0 ", ID)
                DataReader = SQLCommand.ExecuteReader()
                DigitalObjectExists = DataReader.HasRows()

                If Not DigitalObjectExists Then
                    DigitalObjectID = CType(-1, Int64)
                End If

                While DataReader.Read()
                    If DataReader.Item("Active") Then
                        Dim DaoGrpItem As New DaoGrp
                        DaoGrpItem.ComponentID = DataReader.Item("ComponentID")
                        DaoGrpItem.DigitalObjectID = DataReader.Item("DigitalObjectID")
                        DaoGrpItem.DigitalObjectName = DataReader.Item("DOName")
                        DaoGrpItem.FileID = DataReader.Item("FileID")
                        DaoGrpItem.FileName = DataReader.Item("DOFileName")
                        DaoGrpItem.Type = DataReader.Item("Type")
                        DaoGrpItem.ArchiveID = DataReader.Item("ArchiveID")
                        DaoGrpItem.TopographicalQuota = DataReader.Item("TopographicalQuota")

                        If CInt(DataReader.Item("Type")) = 0 Then
                            DaoGrpItem.QuantityOfFiles = GetQuantityOfFiles(DataReader.Item("DigitalObjectID"))
                        Else
                            DaoGrpItem.QuantityOfFiles = 1
                        End If
                        DaoGrp.Add(DaoGrpItem)
                    End If
                End While
                DataReader.Close()

                If (OtherLevel = "D" Or OtherLevel = "DC" Or OtherLevel = "UI") Then
                    IsDSDCUI = True
                Else
                    IsDSDCUI = False
                End If

                Transaction.Commit()
            Catch ex As SqlException
                Transaction.Rollback()
            Catch ex As Exception

            Finally
                Transaction.Dispose()
                DataReader.Close()
                SQLCommand.Dispose()
                SQLConnection.Close()
                SQLConnection.Dispose()
            End Try
        End Sub



        Public Overrides Function Children() As LazyNodeCollection
            Dim SQLConnection As SQLConnection = New SQLConnection(myConnectionString)
            Dim SQLCommand As New SQLCommand
            SQLCommand.Connection = SQLConnection
            Dim DataReader As SqlDataReader
            Dim NodeCollection As New LazyNodeCollection
            Try
                SQLConnection.Open()

                Select Case SortBy
                    Case LazyNode.SortingStyles.ByUnitDateInitial
                        SQLCommand.CommandText = String.Format("SELECT ID FROM Components WHERE ParentID = '{0}' ORDER BY UnitDateInitial, UnitID", myID)
                    Case LazyNode.SortingStyles.ByUnitID
                        SQLCommand.CommandText = String.Format("SELECT ID FROM Components WHERE ParentID = '{0}' ORDER BY UnitID, UnitDateInitial", myID)
                    Case LazyNode.SortingStyles.ByOtherLevel
                        SQLCommand.CommandText = String.Format("SELECT ID FROM Components WHERE ParentID = '{0}' ORDER BY OtherLevel, UnitID, UnitDateInitial", myID)
                    Case Else
                        SQLCommand.CommandText = String.Format("SELECT ID, UnitTitle, UnitdateInitial, UnitdateFinal FROM Components WHERE ParentID = '{0}'", myID)
                End Select

                DataReader = SQLCommand.ExecuteReader()
                While (DataReader.Read)
                    Dim ID As Integer = DBToInt(DataReader.Item("ID"))
                    Dim Node As New SQLLazyNode(ID, Capacities.Minimum)
                    'Node.SortBy = Me.SortBy
                    NodeCollection.Add(Node)
                End While

            Catch ex As Exception
                'MsgBoxException(ex)
            Finally
                SQLCommand.Dispose()
                DataReader.Close()
                SQLConnection.Close()
                SQLConnection.Dispose()
            End Try
            Return NodeCollection
        End Function


        Public Overrides ReadOnly Property HasChildren() As Boolean
            Get
                Dim SQLConnection As SQLConnection = New SQLConnection(myConnectionString)
                Dim SQLCommand As New SQLCommand
                SQLCommand.Connection = SQLConnection
                Dim ChildrenAvailable As Boolean

                Try
                    SQLConnection.Open()
                    SQLCommand.CommandText = String.Format("SELECT HasChildren FROM Components WHERE ID = '{0}'", myID)
                    ChildrenAvailable = DBToBool(SQLCommand.ExecuteScalar)
                Catch ex As Exception
                    'MsgBoxException(ex)
                Finally
                    SQLCommand.Dispose()
                    SQLConnection.Close()
                    SQLConnection.Dispose()
                End Try
                Return ChildrenAvailable
            End Get
        End Property


        Public Overrides Function colParentsToNItself() As LazyNodeCollection
            Dim NodeCollection As New LazyNodeCollection
            Dim ID As Integer = DBToInt(myID)
            While ID <> 0
                Dim Node As New SQLLazyNode(ID, Capacities.Minimum)
                NodeCollection.Add(Node)
                ID = DBToInt(Node.ParentID)
            End While
            Return NodeCollection
        End Function

        Public Overloads Overrides Function strFullPath() As String
            Return strFullPath(colParentsToNItself())
        End Function


        Public Overloads Overrides Function strFullPath(ByVal colParentsToNItself As LazyNodeCollection) As String
            Dim Path As String

            Dim iFirst As Integer = colParentsToNItself.Count - 1
            Dim iLast As Integer = 0
            While iFirst >= iLast
                If iFirst = 0 Then
                    Path &= colParentsToNItself.Item(iFirst).UnitId
                Else
                    If colParentsToNItself.Item(iFirst).OtherLevel = colParentsToNItself.Item(iFirst - 1).OtherLevel.Remove(0, 1) Then
                        Path &= colParentsToNItself.Item(iFirst).UnitId & "-"
                    Else
                        Path &= colParentsToNItself.Item(iFirst).UnitId & "/"
                    End If
                End If

                iFirst = iFirst - 1
            End While

            Return Path
        End Function




        '*************************************************************************************
        ' Function: objQryRecs
        ' Purpose: It returns all the Rec(ord)s ID's derived from the 
        '          sql Q(ue)ry(- gotten for the entrance of text,
        '          given for the user in textboxes, that it is related with the columns of the 
        '          components table)
        '*************************************************************************************
        Public Function objQryRecs(ByVal strQry As String, ByVal strSortBy As String, _
            ByVal strOrder As String) As RecordsIDCollection

            Dim RecordsIDCollection As New RecordsIDCollection
            Dim strCmdText As String ' Com(an)d Text, sqlQuery

            Dim SQLConnection As SQLConnection = New SQLConnection(myConnectionString)

            Dim SQLCommand As New SQLCommand
            SQLCommand.Connection = SQLConnection

            Dim DataReader As SqlDataReader
            Select Case strSortBy
                Case "date"
                    strCmdText = String.Format("{0} ORDER BY UnitDateInitial {1}", strQry, strOrder)
                Case "title"
                    strCmdText = String.Format("{0} ORDER BY UnitTitle {1}", strQry, strOrder)
                Case "level"
                    strCmdText = String.Format("{0} ORDER BY 1 {1}", strOrderByOtherLevelQry(strQry), strOrder)
                Case Else
                    strCmdText = String.Format("{0} ORDER BY CompleteUnitID ASC", strQry)
            End Select
            Try
                SQLConnection.Open()
                SQLCommand.CommandText = String.Format(strCmdText)
                SQLCommand.CommandTimeout = myConnectTimeout
                DataReader = SQLCommand.ExecuteReader()
                While DataReader.Read()
                    RecordsIDCollection.Add(DataReader.Item("ID"))
                End While
            Catch ex As Exception
                Dim erro As String = ex.ToString
                Dim erro1 As String = ex.ToString
            Finally
                SQLCommand.Dispose()
                DataReader.Close()
                SQLConnection.Close()
            End Try
            Return RecordsIDCollection
        End Function

        '*************************************************************************************
        ' Function: sdrOrderByOtherLevelASCQry
        ' Purpose: It returns a String commandText query for order the results from OtherLevel='F'
        '     OtherLevel='D'. I call it ascendant way.
        '*************************************************************************************
        Private Function strOrderByOtherLevelQry(ByVal strSubQry As String) As String
            Dim strOtherLevelAscOrder() As String = {"D", "DC", "UI", "SSSR", "SSR", "SR", "SSSC", "SSC", "SC", "SSF", "SF", "F"}
            Dim strVariables() As String = {"x1", "x2", "x3", "x4", "x5", "x6", "x7", "x8", "x9", "x10", "x11", "x12"}
            Dim i As Integer
            Dim strOtherLevelASCQry As String

            For i = 1 To 12
                strOtherLevelASCQry &= "(SELECT " & i & " AS Col1," & strVariables(i - 1) & ".ID," & strVariables(i - 1) & ".OtherLevel" & _
                    " FROM (" & strSubQry & ") AS " & strVariables(i - 1) & _
                    " WHERE OtherLevel='" & strOtherLevelAscOrder(i - 1) & "')UNION"
            Next
            strOtherLevelASCQry = strOtherLevelASCQry.TrimEnd(CStr("UNION").ToCharArray)
            Return strOtherLevelASCQry
        End Function
#End Region

    End Class

End Namespace


