package pt.keep.objectvault;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.xml.ws.http.HTTPException;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.multipart.ByteArrayPartSource;
import org.apache.commons.httpclient.methods.multipart.FilePart;
import org.apache.commons.httpclient.methods.multipart.MultipartRequestEntity;
import org.apache.commons.httpclient.methods.multipart.Part;
import org.apache.commons.httpclient.methods.multipart.PartSource;
import org.apache.commons.httpclient.methods.multipart.StringPart;
import org.apache.commons.httpclient.params.HttpClientParams;

public class ObjectVaultClient {

	private String serverAddress;

	public ObjectVaultClient(String serverAddress) {
		this.serverAddress = serverAddress;
	}

	public String store(byte[] bytes, String fileName,
			Map<String, String> metadata) throws HttpException, IOException {

		PostMethod filePost = new PostMethod(serverAddress);

		if (metadata == null)
			metadata = new HashMap<String, String>();

		PartSource s;
		int i = 0;
		Part[] parts = new Part[metadata.keySet().size() + 1];
		parts[i++] = new FilePart(fileName, new ByteArrayPartSource(fileName,
				bytes));
		for (String key : metadata.keySet()) {
			parts[i++] = new StringPart(key, metadata.get(key));
		}
		filePost.setRequestEntity(new MultipartRequestEntity(parts, filePost
				.getParams()));

		HttpClient client = new HttpClient();
		HttpClientParams params = new HttpClientParams();
		params.setSoTimeout(1000 * 60 * 20);
		client.setParams(params);

		int statusCode = client.executeMethod(filePost);

		if (statusCode == HttpStatus.SC_OK) {
			String ID = filePost.getResponseBodyAsString();
			filePost.releaseConnection();
			return ID;
		} else {
			filePost.releaseConnection();
			throw new HttpException("" + statusCode);
		}
	}

	public String store(byte[] bytes, String fileName) throws HttpException,
			IOException {
		return store(bytes, fileName, null);
	}

	public String store(File file, Map<String, String> metadata)
			throws HttpException, IOException {

		byte[] buffer = new byte[(int) file.length()];

		FileInputStream fis = new FileInputStream(file);
		fis.read(buffer);
		fis.close();
		return store(buffer, file.getName(), metadata);
	}

	public String store(File file) throws HttpException, IOException {
		return store(file, null);
	}

	public byte[] retrieve(String ID) throws HttpException, IOException {
		// Instantiate an HttpClient
		HttpClient client = new HttpClient();

		// Instantiate a GET HTTP method
		HttpMethod method = new GetMethod(serverAddress);

		// Define name-value pairs to set into the QueryString
		NameValuePair nvp1 = new NameValuePair("GET", ID);
		method.setQueryString(new NameValuePair[] { nvp1 });

		int statusCode = client.executeMethod(method);
		if (statusCode == HttpStatus.SC_OK) {
			byte[] file = method.getResponseBody();
			method.releaseConnection();
			return file;
		} else {
			method.releaseConnection();
			throw new HttpException("" + statusCode);

		}
	}

	public InputStream retrieveAsStream(String ID) throws HttpException,
			IOException {
		// Instantiate an HttpClient
		HttpClient client = new HttpClient();

		// Instantiate a GET HTTP method
		HttpMethod method = new GetMethod(serverAddress);

		// Define name-value pairs to set into the QueryString
		NameValuePair nvp1 = new NameValuePair("GET", ID);
		method.setQueryString(new NameValuePair[] { nvp1 });

		int statusCode = client.executeMethod(method);
		if (statusCode == HttpStatus.SC_OK) {
			InputStream stream = method.getResponseBodyAsStream();
			method.releaseConnection();
			return stream;
		} else {
			method.releaseConnection();
			throw new HttpException("" + statusCode);

		}
	}
	
	public boolean delete(String ID) throws HttpException, IOException {
		// Instantiate an HttpClient
		HttpClient client = new HttpClient();

		// Instantiate a GET HTTP method
		HttpMethod method = new GetMethod(serverAddress);

		// Define name-value pairs to set into the QueryString
		NameValuePair nvp1 = new NameValuePair("DEL", ID);
		method.setQueryString(new NameValuePair[] { nvp1 });

		int statusCode = client.executeMethod(method);
		if (statusCode == HttpStatus.SC_OK) {
			method.getResponseBody();
			method.releaseConnection();
			return true;
		} else {
			method.releaseConnection();
			throw new HTTPException(statusCode);
		}
	}

	public static void saveFile(File fileToCreate, byte[] bytesToSave)
			throws IOException {
		FileOutputStream fos = new FileOutputStream(fileToCreate);
		fos.write(bytesToSave);
		fos.close();
	}

	public static void main(String[] args) throws HttpException, IOException {
		String serverAddress = "http://vaultserver.keep.pt/";
		ObjectVaultClient client = new ObjectVaultClient(serverAddress);
		System.out.print("Uploading file... ");
		String ID = client.store(new File("c:/temp/AutoShutdown.exe"));
		System.out.println("done. ID=" + ID);

		System.out.print("Downloading file... ");
		byte[] bytes = client.retrieve(ID);
		System.out.println("done.");

		System.out.print("Saving file... ");
		saveFile(new File("c:/temp/downloaded.file"), bytes);
		System.out.println("done.");

	}
}

/*
 * PostMethod filePost = new PostMethod(serverAddress); if (metadata == null)
 * metadata = new HashMap<String, String>();
 * 
 * int i = 0; Part[] parts = new Part[metadata.keySet().size() + 1]; parts[i++]
 * = new FilePart(file.getName(), file); for (String key : metadata.keySet()) {
 * parts[i++] = new StringPart(key, metadata.get(key)); }
 * filePost.setRequestEntity(new MultipartRequestEntity(parts, filePost
 * .getParams()));
 * 
 * HttpClient client = new HttpClient(); int statusCode =
 * client.executeMethod(filePost);
 * 
 * if (statusCode == HttpStatus.SC_OK) { String ID =
 * filePost.getResponseBodyAsString(); filePost.releaseConnection(); return ID;
 * } else { filePost.releaseConnection(); throw new HTTPException(statusCode); }
 */