package pt.keep.speedtest;

import java.io.File;
import java.io.IOException;
import java.util.GregorianCalendar;
import java.util.Random;

import org.apache.commons.httpclient.HttpException;

import pt.keep.objectvault.ObjectVaultClient;

/**
 * 
 * @author Miguel Ferreira <mferreira@keep.pt>
 * 
 */
public class SpeedTest {
	private GregorianCalendar startTime;
	private GregorianCalendar finishTime;
	private long trafficLength; // bytes

	public SpeedTest(long trafficLength) {
		start();
		this.trafficLength = trafficLength;
	}

	public void start() {
		startTime = new GregorianCalendar();
	}

	public void finish() {
		finishTime = new GregorianCalendar();
	}

	public double getBandwidth() {
		double timeSpan = (finishTime.getTimeInMillis() - startTime
				.getTimeInMillis()) / 1000.0;
		return trafficLength * 8.0 / timeSpan;
	}

	public static void main(String[] args) throws HttpException, IOException {
		String serverAddress;

		System.out.println("Speed Test Tool by KEEP SOLUTIONS");

		if (args.length != 1) {
			System.err
					.println("Syntax: java -jar speedtest.jar [Object Vault Server URL]");
			System.err
					.println("Using: java -jar speedtest.jar http://sol.tt.pt/objectvaultserver/default.aspx");
			serverAddress = "http://sol.tt.pt/objectvaultserver/default.aspx";
		} else {
			serverAddress = args[0];
		}

		ObjectVaultClient client = new ObjectVaultClient(serverAddress);
	

		byte[] memData = new byte[(int) 1024 * 1024];
		SpeedTest randomUpload = new SpeedTest(memData.length);

		Random rand = new Random(GregorianCalendar.getInstance()
				.getTimeInMillis());
		rand.nextBytes(memData);
		System.out.print("Uploading random file... ");
		randomUpload.start();
		String rID = client.store(memData, "testFile");
		randomUpload.finish();
		System.out.println("done. ID=" + rID);

		SpeedTest randomDownload = new SpeedTest(memData.length);
		System.out.print("Uploading random file... ");
		randomDownload.start();
		client.retrieveAsStream(rID);
		randomDownload.finish();
		System.out.println("done.");


		System.out.print("Deleting uploaded file... ");
		client.delete(rID);
		System.out.println("done.");

		
		System.out.println();
		System.out.println(String.format(
				"Random upload bandwidth (Kb/s) = %1$.1f", randomUpload
						.getBandwidth() / 1024.0));
		System.out.println(String.format(
				"Random download bandwidth (Kb/s) = %1$.1f", randomDownload
						.getBandwidth() / 1024.0));



	}

}
