Imports System.IO
Imports System.net
Imports System.Text
Imports System.Text.Encoding
Imports DigitArq.Data.Vault.Utility


Public Class ObjectVaultClient

#Region "Properties"

    ' Server Address
    Private VaultServerAddress As String

    Private Const VAULT_PROPERTY_ORIGINAL_FILENAME As String = "OriginalFilename"
    Public Const USER_AGENT As String = "ObjectVaultClient.NET 1.1"


    '' Special commands
    'Public Const COMMAND_SECRET As String = "SECRET"
    'Public Const COMMAND_DELETE As String = "DEL"
    'Public Const COMMAND_RETRIEVE As String = "GET"
    'Public Const COMMAND_METADATA As String = "MD"

    '' HTTP error codes
    'Public Const HTTP_STATUS_OK As Integer = 200
    'Public Const HTTP_STATUS_BAD_REQUEST As Integer = 400
    'Public Const HTTP_STATUS_NOT_FOUND As Integer = 404
    'Public Const HTTP_STATUS_INTERNAL_SERVER_ERROR As Integer = 500

#End Region

#Region "Constructor"

    Public Sub New(ByVal VaultServerAddress As String)
        Me.VaultServerAddress = VaultServerAddress
    End Sub
#End Region

#Region "Store methods"

    'Stores file in the vault
    Public Function Store(ByVal Data As Byte(), _
                          ByVal Filename As String, _
                          ByVal Metadata As IDictionary) As Byte()
        Dim ByteArrayStream As New ByteArrayStream(Data)
        Dim result As Byte() = Store(ByteArrayStream, Filename, Metadata, Nothing)
        ByteArrayStream.Close()
        ByteArrayStream = Nothing
        Return result
    End Function


    'Stores file in the vault
    Public Function Store(ByVal DataStream As Stream, _
                          ByVal Filename As String, _
                          ByVal Metadata As IDictionary) As Byte()

        Return Store(DataStream, Filename, Metadata, Nothing)
    End Function


    'Stores file in the vault
    Public Function Store(ByVal DataStream As Stream, _
                     ByVal Filename As String, _
                     ByVal Metadata As IDictionary, _
                     ByVal Secret As String, _
                     Optional ByVal DownloadLimit As Integer = -1) As Byte()

        If Metadata Is Nothing Then Metadata = New Hashtable

        If Not Secret Is Nothing Then Metadata.Add(COMMAND_SECRET, Secret)
        If DownloadLimit > 0 Then Metadata.Add(COMMAND_DOWNLOAD_LIMIT, CStr(DownloadLimit))

        Dim response As Byte() = PostData(DataStream, Path.GetFileName(Filename), Metadata)
        Return response

    End Function




#End Region

#Region "Retrieve metadata"




    Public Function RetrieveMetadata(ByVal ID As String) As IDictionary
        If ID Is Nothing OrElse ID = "" Then
            Throw New VaultException(HTTP_STATUS_BAD_REQUEST, "Invalid ID")
        End If

        Try
            Return Utils.LoadCustomMetadata(RetrieveMetadataAsXML(ID))
        Catch ex As Exception
            Throw New VaultException(HTTP_STATUS_INTERNAL_SERVER_ERROR, ex.Message)
        End Try
    End Function


    Public Function RetrieveMetadataAsXML(ByVal ID As String) As String
        If ID Is Nothing OrElse ID = "" Then
            Throw New VaultException(HTTP_STATUS_BAD_REQUEST, "Invalid ID")
        End If

        Try

            Dim WebClient As New WebClient
            WebClient.QueryString.Add("MD", ID)
            Dim ResponseBuffer() As Byte = WebClient.DownloadData(VaultServerAddress)

            Return Encoding.UTF8.GetString(ResponseBuffer).Substring(1)

        Catch ex As Exception
            Throw New VaultException(HTTP_STATUS_INTERNAL_SERVER_ERROR, ex.Message)
        End Try
    End Function

#End Region

#Region "Retrive objects"


    Public Function Retrieve(ByVal ID As String) As Byte()
        Return Retrieve(ID, Nothing)
    End Function


    'Public Function Retrieve(ByVal ID As String) As Byte()
    '    If ID Is Nothing OrElse ID = "" Then
    '        Throw New VaultException(HTTP_STATUS_BAD_REQUEST, "Invalid ID")
    '    End If

    '    Try
    '        Dim WebClient As New WebClient
    '        WebClient.QueryString.Add(COMMAND_RETRIEVE, ID)
    '        Return WebClient.DownloadData(VaultServerAddress)
    '    Catch ex As Exception
    '        Throw New VaultException(HTTP_STATUS_INTERNAL_SERVER_ERROR, "Unknown error while retrieving file")
    '    End Try

    'End Function



    Public Function Retrieve(ByVal ID As String, ByVal Secret As String) As Byte()
        If ID Is Nothing OrElse ID = "" Then
            Throw New VaultException(HTTP_STATUS_BAD_REQUEST, "Invalid ID")
        End If

        Try
            Dim WebClient As New WebClient
            WebClient.QueryString.Add(COMMAND_RETRIEVE, ID)
            If (Not Secret Is Nothing) Then WebClient.QueryString.Add(COMMAND_SECRET, Secret)

            Dim resultStream As Stream = WebClient.OpenRead(VaultServerAddress)
            Dim MemoryStream As New MemoryStream
            Utils.CopyStream(resultStream, MemoryStream)
            Dim Buffer As Byte() = MemoryStream.ToArray
            MemoryStream.Close()
            resultStream.Close()
            System.GC.Collect()
            Return Buffer
        Catch ex As Exception
            Throw New VaultException(HTTP_STATUS_INTERNAL_SERVER_ERROR, "Unknown error while retrieving file")
        End Try

    End Function

#End Region

#Region "Head"


    Public Function Head() As Byte()
        Try
            Dim Request As HttpWebRequest = WebRequest.Create(VaultServerAddress)

            Request.Method = "GET"
            Request.KeepAlive = True
            Request.Credentials = System.Net.CredentialCache.DefaultCredentials()

            Dim Response As WebResponse = Request.GetResponse()
            Dim ResponseBuffer(Response.ContentLength - 1) As Byte
            Response.GetResponseStream.Read(ResponseBuffer, 0, Response.ContentLength)
            Return ResponseBuffer
        Catch ex As Exception
            Throw New VaultException(HTTP_STATUS_INTERNAL_SERVER_ERROR, ex.Message)
        End Try
    End Function

#End Region

#Region "Delete object"

    Public Function Delete(ByVal ID As String) As Boolean
        Return Delete(ID, Nothing)
    End Function


    Public Function Delete(ByVal ID As String, ByVal Secret As String) As Boolean
        If ID Is Nothing OrElse ID = "" Then
            Throw New VaultException(HTTP_STATUS_BAD_REQUEST, "ID is missing")
        End If

        Try

            Dim Request As HttpWebRequest

            If Secret Is Nothing OrElse Secret.Length = 0 Then
                Request = WebRequest.Create(VaultServerAddress & "?DEL=" & ID)
            Else
                Request = WebRequest.Create(VaultServerAddress & "?DEL=" & ID & "&SECRET=" & Secret)
            End If


            Request.Method = "GET"
            Request.UserAgent = USER_AGENT
            Request.KeepAlive = True
            Request.Credentials = System.Net.CredentialCache.DefaultCredentials()

            Dim Response As WebResponse = Request.GetResponse()
            ' success
            Return True
        Catch ex As Exception
            Throw New VaultException(HTTP_STATUS_INTERNAL_SERVER_ERROR, ex.Message)
        End Try
    End Function

#End Region

#Region "Helper methods"


    Private Function PostData(ByVal Bytestream As Stream, ByVal Filename As String, ByVal CustomMetadata As IDictionary) As Byte()


        If (Bytestream.Length > MAXIMUM_FILE_SIZE) Then
            Throw New VaultFileSizeException(HTTP_STATUS_BAD_REQUEST, "Maximum file size exceeded. Maximum file size is set to " & MAXIMUM_FILE_SIZE & " bytes")
        End If

        Try

            Dim boundary As String = "---------------------------" & DateTime.Now.Ticks.ToString("x")
            Dim Request As HttpWebRequest = WebRequest.Create(VaultServerAddress)

            Request.UserAgent = USER_AGENT
            Request.ContentType = "multipart/form-data; boundary=" & boundary
            Request.Credentials = System.Net.CredentialCache.DefaultCredentials()
            Request.Method = "POST"
            Request.KeepAlive = False

            'Dim MemStream As Stream = New System.IO.MemoryStream
            Dim RequestStream As Stream = Request.GetRequestStream()

            Dim FormDataTemplate As String = vbCrLf & "--" & boundary & vbCrLf & "Content-Disposition: form-data; name=""{0}""" & vbCrLf & vbCrLf & "{1}"

            For Each Key As String In CustomMetadata.Keys
                Dim FormItem As String = String.Format(FormDataTemplate, Key, CustomMetadata(Key))
                Dim FormItemBytes() As Byte = ASCII.GetBytes(FormItem)
                RequestStream.Write(FormItemBytes, 0, FormItemBytes.Length)
            Next

            Dim BoundaryBytes() As Byte = ASCII.GetBytes(vbCrLf & "--" & boundary & vbCrLf)
            RequestStream.Write(BoundaryBytes, 0, BoundaryBytes.Length)

            Dim FileDataTemplate As String = "Content-Disposition: form-data; name=""{0}""; filename=""{1}""" & vbCrLf & "Content-Type: application/octet-stream" & vbCrLf & vbCrLf
            Dim FileDataItem As String = String.Format(FileDataTemplate, "file1_", Filename)
            Dim FileDataItemBytes() As Byte = ASCII.GetBytes(FileDataItem)

            RequestStream.Write(FileDataItemBytes, 0, FileDataItemBytes.Length)


            ' seek the beginning of the stream
            Bytestream.Seek(0, SeekOrigin.Begin)
            ' write file to stream
            'Dim BUFFER_SIZE As Integer = 1024 * 1024 ' 1 MB
            'Dim Buffer(BUFFER_SIZE) As Byte
            'Do
            '    Dim BytesCopied As Integer = Bytestream.Read(Buffer, 0, BUFFER_SIZE)
            '    If BytesCopied = 0 Then Exit Do
            '    RequestStream.Write(Buffer, 0, BytesCopied)
            'Loop
            'Buffer = Nothing
            Utils.CopyStream(Bytestream, RequestStream)

            '-- Commented by lmferros
            'Bytestream.Flush()
            'Bytestream.Close()
            'Bytestream = Nothing
            '--

            Dim EndBoundaryBytes() As Byte = ASCII.GetBytes(vbCrLf & "--" & boundary & "--" & vbCrLf)
            RequestStream.Write(EndBoundaryBytes, 0, EndBoundaryBytes.Length)


            RequestStream.Flush()
            RequestStream.Close()

            Request.Timeout = 3600000
            Dim WebResponse As WebResponse = Request.GetResponse()
            Dim StreamReader As New StreamReader(WebResponse.GetResponseStream)
            Dim ObjectID As String = StreamReader.ReadToEnd

            StreamReader.DiscardBufferedData()
            StreamReader.Close()
            StreamReader = Nothing
            RequestStream = Nothing
            WebResponse = Nothing

            System.GC.Collect()
            Return ASCII.GetBytes(ObjectID)
        Catch ex As OutOfMemoryException
            Throw New VaultException(HTTP_STATUS_INTERNAL_SERVER_ERROR, ex.Message & vbNewLine & vbNewLine & ex.StackTrace)
        Catch ex As Exception
            Throw New VaultException(HTTP_STATUS_INTERNAL_SERVER_ERROR, ex.Message & vbNewLine & vbNewLine & ex.StackTrace)
        End Try
    End Function


#End Region

End Class
