Imports System.IO
Imports DigitArq.DataServices.ObjectVaultServer
Imports DigitArq.Data.Vault.Utility
Imports System.Xml


Public Class ObjectVaultServer
    Inherits System.Web.UI.Page



#Region " Web Form Designer Generated Code "

    'This call is required by the Web Form Designer.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()

    End Sub

    'NOTE: The following placeholder declaration is required by the Web Form Designer.
    'Do not delete or move it.
    Private designerPlaceholderDeclaration As System.Object

    Private Sub Page_Init(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Init
        'CODEGEN: This method call is required by the Web Form Designer
        'Do not modify it using the code editor.
        InitializeComponent()
    End Sub

#End Region

#Region "Constants"

    ' Vault settings
    Private Const MD5_PARTITIONS As Integer = 2
    Private Shared BASE_DIR As String

    Public Shared VAULT_VERSION
    Public Shared HEAD_MESSAGE As String

    Public Sub New()
        BASE_DIR = GetConfig("storage.location")
        VAULT_VERSION = GetConfig("vault.version")
        HEAD_MESSAGE = GetConfig("head.message")
    End Sub

#End Region

#Region "Routing"


    Private Sub Page_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Select Case Request.HttpMethod.ToUpper
            Case HTTP_METHOD_POST
                doPost(Request, Response)
            Case HTTP_METHOD_PUT
                doPut(Request, Response)
            Case HTTP_METHOD_GET
                doGet(Request, Response)
            Case HTTP_METHOD_DELETE
                doDelete(Request, Response)
            Case HTTP_METHOD_HEAD
                doHead(Request, Response)
            Case Else
                doHead(Request, Response)
        End Select

    End Sub

#End Region

#Region "Request handlers"




    ' Method that handles requests 
    Sub doGet(ByRef Request As HttpRequest, ByRef Response As HttpResponse)

        If Request.QueryString.Get(COMMAND_RETRIEVE) = "" AndAlso _
           Request.QueryString.Get(COMMAND_METADATA) = "" AndAlso _
           Request.QueryString.Get(COMMAND_DELETE) = "" Then
            doHead(Request, Response)
        End If

        Dim Secret As String = Request.QueryString.Get(COMMAND_SECRET)

        If Request.QueryString.Get(COMMAND_RETRIEVE) <> "" Then
            Dim ObjectID As String = Request.QueryString.Get(COMMAND_RETRIEVE).ToUpper
            HandleRetrieve(ObjectID, Secret)
        ElseIf Request.QueryString.Get(COMMAND_METADATA) <> "" Then
            Dim MetadataID As String = Request.QueryString.Get(COMMAND_METADATA).ToUpper
            HandleRetrieveMetadata(MetadataID, Secret)
        ElseIf Request.QueryString.Get(COMMAND_DELETE) <> "" Then
            Dim ObjectID As String = Request.QueryString.Get(COMMAND_DELETE).ToUpper
            HandleDelete(ObjectID, Secret)
        Else
            ErrorResponse(HTTP_STATUS_BAD_REQUEST, "Object or metadata ID not specified")
            Exit Sub
        End If

    End Sub




    ' handle PUT and POST
    Sub doPut(ByRef Request As HttpRequest, ByRef Response As HttpResponse)
        Try
            ' debug line
            'Request.SaveAs("e:\requests\" + Path.GetFileName(Path.GetTempFileName), True)

            ' if diff than one file exit
            If (Request.Files.Count <> 1) Then
                ErrorResponse(HTTP_STATUS_BAD_REQUEST, "No files or more than one file included in request")
                Exit Sub
            End If

            Dim HttpPostedFile As HttpPostedFile = Request.Files.Item(0)
            'Changed from MD5 to a Standard Hash function (e.g. SHA1). Handles bigger files
            Dim ID As String = Utils.Hash(HttpPostedFile.InputStream).ToUpper

            Dim ObjectAbsoluteFilename As String = GetObjectAbsolutePath(ID)
            Dim CustomMetadataAbsoluteFilename As String = GetCustomMetadataAbsolutePath(ID)
            Dim VaultMetadataAbsoluteFilename As String = GetVaultMetadataAbsolutePath(ID)

            ' create directory
            Dim ObjectDirectory As String = GetObjectDirectory(ID)
            Utils.CreateDirectory(ObjectDirectory)

            'save object
            HttpPostedFile.SaveAs(ObjectAbsoluteFilename)

            'save metadata
            Dim CustomMetadata As IDictionary = New Hashtable
            For Each key As String In Request.Form
                If (key.ToUpper <> COMMAND_SECRET AndAlso key.ToUpper <> COMMAND_DOWNLOAD_LIMIT) Then
                    CustomMetadata.Add(key, Request.Form.Get(key))
                End If
            Next

            Utils.SaveMetadata(CustomMetadataAbsoluteFilename, CUSTOM_METADATA_ROOT_ELEMENT, CustomMetadata)

            ' automatically generated metadata
            Dim VaultMetadata As IDictionary = New Hashtable
            VaultMetadata.Add("Filename", HttpPostedFile.FileName)
            VaultMetadata.Add("Length", HttpPostedFile.ContentLength)
            VaultMetadata.Add("MD5", ID)
            VaultMetadata.Add("Date", DateTime.Now.ToShortDateString)
            VaultMetadata.Add("Time", DateTime.Now.ToLongTimeString)
            VaultMetadata.Add("UserHostAddress", Request.UserHostAddress)
            VaultMetadata.Add("UserHostName", Request.UserHostName)
            VaultMetadata.Add("UserAgent", Request.UserAgent)

            If (Not Request.Form.Get(COMMAND_SECRET) Is Nothing) AndAlso Request.Form.Get(COMMAND_SECRET) <> "" Then
                VaultMetadata.Add(VAULT_METADATA_SECRET, Request.Form.Get(COMMAND_SECRET))
            End If

            If (Not Request.Form.Get(COMMAND_DOWNLOAD_LIMIT) Is Nothing) AndAlso Request.Form.Get(COMMAND_DOWNLOAD_LIMIT) <> "" Then
                VaultMetadata.Add(VAULT_METADATA_DOWNLOAD_LIMIT, Request.Form.Get(COMMAND_DOWNLOAD_LIMIT))
            End If

            Utils.SaveMetadata(VaultMetadataAbsoluteFilename, VAULT_METADATA_ROOT_ELEMENT, VaultMetadata)

            HttpPostedFile.InputStream.Close()

            ' prepare reply
            Response.Clear()
            Response.ContentType = "text/plain"
            Response.StatusCode = HTTP_STATUS_OK
            Response.StatusDescription = "The object was stored in the vault"
            Response.Write(ID)
            System.GC.Collect()


        Catch ex As Exception
            ErrorResponse(HTTP_STATUS_INTERNAL_SERVER_ERROR, ex.Message & vbNewLine & ex.StackTrace)
        End Try


    End Sub

#End Region

#Region "Handlers"

    Private Sub HandleDelete(ByVal ID As String, ByVal Secret As String)
        Dim ObjectDirectory As String = GetObjectDirectory(ID)

        If Not Directory.Exists(ObjectDirectory) Then
            ErrorResponse(HTTP_STATUS_NOT_FOUND, "Object not found")
            Exit Sub
        End If

        ' If no secret or secret is correct send file to client
        If GetSecret(ID) Is Nothing OrElse GetSecret(ID).Equals(Secret) Then
            Directory.Delete(ObjectDirectory, True)
            Response.Clear()
            Response.ContentType = "text/plain"
            Response.Write("OBJECT REMOVED")
            Response.End()
        Else
            ErrorResponse(HTTP_STATUS_FORBIDDEN, "Forbidden")
        End If
    End Sub

    Private Sub HandleRetrieveMetadata(ByVal ID As String, ByVal Secret As String)
        Dim CustomMetadataAbsoluteFilename As String = GetCustomMetadataAbsolutePath(ID)

        If Not File.Exists(CustomMetadataAbsoluteFilename) Then
            ErrorResponse(HTTP_STATUS_NOT_FOUND, "Metadata not found")
            Exit Sub
        End If

        ' If no secret or secret is correct send file to client
        If GetSecret(ID) Is Nothing OrElse GetSecret(ID).Equals(Secret) Then
            Response.Clear()
            Response.AddHeader("Content-Disposition", "attachment; filename=" + "metadata.xml")
            Response.ContentType = "application/xml"
            Response.WriteFile(CustomMetadataAbsoluteFilename)
            Response.End()
        Else
            ErrorResponse(HTTP_STATUS_FORBIDDEN, "Forbidden")
        End If
    End Sub

    Private Sub HandleRetrieve(ByVal ID As String, ByVal Secret As String)

        Dim ObjectAbsoluteFilename As String = GetObjectAbsolutePath(ID)

        If Not File.Exists(ObjectAbsoluteFilename) Then
            ErrorResponse(HTTP_STATUS_NOT_FOUND, "Object not found")
            Exit Sub
        End If


        If DownloadLimitExceeded(ID) Then
            ErrorResponse(HTTP_STATUS_FORBIDDEN, "Download limit exceeded")
            Exit Sub
        End If


        ' If no secret or secret is correct send file to client
        If GetSecret(ID) Is Nothing OrElse GetSecret(ID).Equals(Secret) Then
            DecrementDownloadLimit(ID)
            Dim Filename As String = GetFilenameFromMetadata(ID)
            Response.Clear()
            Response.AddHeader("Content-Disposition", "attachment; filename=" & Server.UrlEncode(Filename))
            Response.ContentType = "application/octet-stream"
            Response.WriteFile(ObjectAbsoluteFilename)
            Response.End()
        Else
            ErrorResponse(HTTP_STATUS_FORBIDDEN, "Wrong secret")
        End If
    End Sub






#End Region

#Region "Rerouting Handlers"

    Sub doDelete(ByRef Request As HttpRequest, ByRef Response As HttpResponse)
        'nothing to do
    End Sub



    Sub doHead(ByRef Request As HttpRequest, ByRef Response As HttpResponse)
        Response.Clear()
        Response.ContentType = "text/plain"
        Response.Write(HEAD_MESSAGE)

        Dim username As String = System.Security.Principal.WindowsIdentity.GetCurrent().Name
        Response.Write(vbNewLine & username)
        Response.End()
    End Sub



    Sub doPost(ByRef Request As HttpRequest, ByRef Response As HttpResponse)
        doPut(Request, Response)
    End Sub


    Sub ErrorResponse(ByVal StatusCode As Integer, ByVal Description As String)
        Response.Clear()
        Response.ContentType = "text/plain"
        Response.StatusCode = StatusCode
        'Response.StatusDescription = Description
        Response.Write(Description)
        Response.End()
        System.GC.Collect()
    End Sub

#End Region

#Region "Metadata-related methods"

    Private Function DownloadLimitExceeded(ByVal ID As String) As Integer
        Dim DownloadLimit As String = Utils.LoadVaultMetadataFromFile(GetVaultMetadataAbsolutePath(ID)).Item(VAULT_METADATA_DOWNLOAD_LIMIT)

        If DownloadLimit Is Nothing OrElse DownloadLimit = "" Then
            Return False
        ElseIf CInt(DownloadLimit) = 0 Then
            Return True
        Else
            Return False
        End If
    End Function

    Private Sub DecrementDownloadLimit(ByVal ID As String)
        Dim VaultMetadataAbsoluteFilename As String = GetVaultMetadataAbsolutePath(ID)

        Dim VaultMetadata As IDictionary = Utils.LoadVaultMetadataFromFile(VaultMetadataAbsoluteFilename)
        Dim DownloadLimit As String = VaultMetadata.Item(VAULT_METADATA_DOWNLOAD_LIMIT)

        If Not DownloadLimit Is Nothing AndAlso DownloadLimit <> "" AndAlso CInt(DownloadLimit) > 0 Then
            VaultMetadata.Item(VAULT_METADATA_DOWNLOAD_LIMIT) = CStr(CInt(DownloadLimit) - 1)
            Utils.SaveMetadata(VaultMetadataAbsoluteFilename, VAULT_METADATA_ROOT_ELEMENT, VaultMetadata)
        End If
    End Sub



    Private Function GetSecret(ByVal ID As String) As String
        Dim Secret As String = Utils.LoadVaultMetadataFromFile(GetVaultMetadataAbsolutePath(ID)).Item(VAULT_METADATA_SECRET)
        If Secret Is Nothing OrElse Secret = "" Then Return Nothing Else Return Secret
    End Function


    Private Function GetVaultMetadata(ByVal ID As String) As IDictionary
        Return Utils.LoadVaultMetadataFromFile(GetVaultMetadataAbsolutePath(ID))
    End Function


    Private Function GetCustomMetadata(ByVal ID As String) As IDictionary
        Return Utils.LoadCustomMetadataFromFile(GetCustomMetadataAbsolutePath(ID))
    End Function




    Private Function GetFilenameFromMetadata(ByVal ID As String) As String
        Return Utils.LoadVaultMetadataFromFile(GetVaultMetadataAbsolutePath(ID)).Item("Filename")
    End Function



#End Region

#Region "Directories and Filenames"

    Private Function GetObjectDirectory(ByVal ID As String) As String
        Return BASE_DIR & String.Join(Path.DirectorySeparatorChar, Utils.Split(ID, MD5_PARTITIONS))
    End Function


    Private Function GetObjectAbsolutePath(ByVal ID As String) As String
        Return GetObjectDirectory(ID) & Path.DirectorySeparatorChar & ID & DATA_FILENAME_SUFFIX
    End Function


    Private Function GetVaultMetadataAbsolutePath(ByVal ID As String) As String
        Return GetObjectDirectory(ID) & Path.DirectorySeparatorChar & ID & VAULT_METADATA_FILENAME_SUFFIX
    End Function


    Private Function GetCustomMetadataAbsolutePath(ByVal ID As String) As String

        Return GetObjectDirectory(ID) & Path.DirectorySeparatorChar & ID & CUSTOM_METADATA_FILENAME_SUFFIX
    End Function

#End Region

#Region "Configuration files"

    Public Function GetConfig(ByVal strKey As String) As String
        Return ConfigurationSettings.AppSettings(strKey)
    End Function

#End Region


End Class
