Imports System.Security.Cryptography
Imports System.Text
Imports System.Text.RegularExpressions
Imports System.IO
Imports System.Xml
Imports System.Collections.Specialized


Public Class Utils

    Public Shared Sub CopyStream(ByVal Input As Stream, ByVal Output As Stream)
        Dim BUFFER_SIZE As Integer = 40 * 1024  ' 1 MB
        Dim Buffer(BUFFER_SIZE - 1) As Byte
        Do
            Dim BytesCopied As Integer = Input.Read(Buffer, 0, BUFFER_SIZE)
            If BytesCopied = 0 Then Exit Do
            Output.Write(Buffer, 0, BytesCopied)
        Loop
        Buffer = Nothing
    End Sub

    Public Shared Function MD5(ByVal ByteStream As Stream) As String
        Dim HashEngine As MD5 = New MD5CryptoServiceProvider
        Dim md5sum() As Byte = HashEngine.ComputeHash(ByteStream)
        HashEngine.Clear()
        Return BitConverter.ToString(md5sum).Replace("-", "")
    End Function



    Public Shared Function MD5(ByRef ByteArray() As Byte) As String
        Dim HashEngine As MD5 = New MD5CryptoServiceProvider
        Dim md5sum() As Byte = HashEngine.ComputeHash(ByteArray)
        HashEngine.Clear()
        Return BitConverter.ToString(md5sum).Replace("-", "")
    End Function


    Public Shared Function SHA1(ByRef ByteArray() As Byte) As String
        Dim HashEngine As New SHA1Managed
        Dim checksum() As Byte = HashEngine.ComputeHash(ByteArray)
        HashEngine.Clear()
        Return BitConverter.ToString(checksum).Replace("-", "")
    End Function



    Public Shared Function SHA1(ByVal ByteStream As Stream) As String
        Dim HashEngine As New SHA1Managed
        Dim checksum() As Byte = HashEngine.ComputeHash(ByteStream)
        HashEngine.Clear()
        Return BitConverter.ToString(checksum).Replace("-", "")
    End Function


    Public Shared Function Hash(ByRef ByteArray() As Byte, ByVal HashEngine As HashAlgorithm) As String
        Dim checksum() As Byte = HashEngine.ComputeHash(ByteArray)
        HashEngine.Clear()
        Return BitConverter.ToString(checksum).Replace("-", "")
    End Function


    Public Shared Function Hash(ByVal ByteStream As Stream, ByVal HashEngine As HashAlgorithm) As String
        Dim checksum() As Byte = HashEngine.ComputeHash(ByteStream)
        HashEngine.Clear()
        Return BitConverter.ToString(checksum).Replace("-", "")
    End Function


    Public Shared Function Hash(ByVal ByteStream As Stream) As String
        Return Hash(ByteStream, New MD5CryptoServiceProvider)
    End Function

    Public Shared Function Hash(ByRef ByteArray() As Byte) As String
        Return Hash(ByteArray, New MD5CryptoServiceProvider)
    End Function



    Public Shared Function Split(ByVal str As String, ByVal value As Integer) As String()
        If str.Length <= value * 3 Then
            Return New String() {str}
        Else
            Dim ret(4) As String
            ret(0) = str.Substring(0, value)
            ret(1) = str.Substring(value, value)
            ret(2) = str.Substring(value * 2, value)
            ret(3) = str.Substring(value * 3)
            Return ret
        End If
    End Function


    'Public Shared Function Split(ByVal str As String, ByVal value As Integer) As String()
    '    Dim ret(value - 1) As String
    '    Dim count As Integer = Math.Round(str.Length / value)
    '    For i As Integer = 0 To value - 2
    '        ret(i) = str.Substring(i * count, count)
    '    Next
    '    ret(value - 1) = str.Substring((value - 1) * count, str.Length - (value - 1) * count)
    '    Return ret
    'End Function



    Public Shared Sub CreateDirectory(ByVal Path As String)
        If Not Directory.Exists(Path) Then
            Directory.CreateDirectory(Path)
        End If
    End Sub




    Public Shared Function SaveMetadata(ByVal MetadataFileName As String, ByVal RootElementName As String, _
                                        ByVal Metadata As IDictionary) As XmlDocument

        'Dim Node As XmlNode
        Dim myMetadataXml As New XmlDocument

        Dim dec As XmlDeclaration = myMetadataXml.CreateXmlDeclaration("1.0", "utf-8", Nothing)
        myMetadataXml.AppendChild(dec)

        Dim Root As XmlElement = myMetadataXml.CreateElement(RootElementName)
        myMetadataXml.AppendChild(Root)

        For Each key As String In Metadata.Keys
            Dim PropertyNode As XmlNode = myMetadataXml.CreateElement("property")
            Dim NameAttribute As XmlNode = myMetadataXml.CreateAttribute("name")
            NameAttribute.Value = key
            PropertyNode.Attributes.Append(NameAttribute)
            PropertyNode.InnerText = Metadata.Item(key)
            Root.AppendChild(PropertyNode)
        Next

        Dim writer As XmlTextWriter = New XmlTextWriter(MetadataFileName, Text.Encoding.UTF8)
        writer.Formatting = Formatting.Indented
        myMetadataXml.Save(writer)
        writer.Close()
        Return myMetadataXml
    End Function



    Public Shared Function ReadBinaryFile(ByVal filename As String) As Byte()
        Dim fs As New FileStream(filename, FileMode.Open)
        Dim br As New BinaryReader(fs)
        Dim ret() As Byte = br.ReadBytes(fs.Length)
        fs.Close()
        br.Close()

        Return ret
    End Function

    Public Shared Function ReadTextFile(ByVal filename As String) As String
        Dim sr As StreamReader = New StreamReader(filename)
        Dim ret As New StringBuilder
        Dim line As String
        Do
            line = sr.ReadLine()
            ret.Append(line)
        Loop Until line Is Nothing
        sr.Close()
        Return ret.ToString
    End Function

    Public Shared Function WriteBinaryFile(ByVal filename As String, ByVal data As Byte()) As Boolean
        Dim writtenFile As Boolean = True
        Try
            Dim fs As New FileStream(filename, FileMode.Create)
            Dim bw As BinaryWriter = New BinaryWriter(fs)
            bw.Write(data, 0, data.Length)
            fs.Close()
            bw.Close()
        Catch ex As Exception
            writtenFile = False
        End Try
        Return writtenFile
    End Function



#Region "Metadata related methods"
    Public Shared Function LoadVaultMetadata(ByVal MetadataXML As String) As IDictionary
        Dim VaultMetadata As New Hashtable

        Dim myMetadataXml As New XmlDocument
        myMetadataXml.LoadXml(MetadataXML)

        Dim root As XmlNode = myMetadataXml.Item(VAULT_METADATA_ROOT_ELEMENT)

        For Each VaultProperty As XmlNode In root.ChildNodes
            If (VaultProperty.Name.Equals("property")) Then
                VaultMetadata.Add(VaultProperty.Attributes("name").Value, VaultProperty.InnerText)
            End If
        Next
        Return VaultMetadata
    End Function


    Public Shared Function LoadVaultMetadataFromFile(ByVal MetadataFilename As String) As IDictionary
        Dim VaultMetadata As New Hashtable

        Dim myMetadataXml As New XmlDocument
        myMetadataXml.Load(MetadataFilename)

        Dim root As XmlNode = myMetadataXml.Item(VAULT_METADATA_ROOT_ELEMENT)

        For Each VaultProperty As XmlNode In root.ChildNodes
            If (VaultProperty.Name.Equals("property")) Then
                VaultMetadata.Add(VaultProperty.Attributes("name").Value, VaultProperty.InnerText)
            End If
        Next
        Return VaultMetadata
    End Function

    Public Shared Function LoadCustomMetadata(ByVal MetadataXML As String) As IDictionary
        Dim VaultMetadata As New Hashtable

        Dim myMetadataXml As New XmlDocument
        myMetadataXml.LoadXml(MetadataXML)

        Dim root As XmlNode = myMetadataXml.Item(CUSTOM_METADATA_ROOT_ELEMENT)

        For Each VaultProperty As XmlNode In root.ChildNodes
            If (VaultProperty.Name.Equals("property")) Then
                VaultMetadata.Add(VaultProperty.Attributes("name").Value, VaultProperty.InnerText)
            End If
        Next
        Return VaultMetadata
    End Function


    Public Shared Function LoadCustomMetadataFromFile(ByVal MetadataFilename As String) As IDictionary
        Dim VaultMetadata As New Hashtable

        Dim myMetadataXml As New XmlDocument
        myMetadataXml.Load(MetadataFilename)

        Dim root As XmlNode = myMetadataXml.Item(CUSTOM_METADATA_ROOT_ELEMENT)

        For Each VaultProperty As XmlNode In root.ChildNodes
            If (VaultProperty.Name.Equals("property")) Then
                VaultMetadata.Add(VaultProperty.Attributes("name").Value, VaultProperty.InnerText)
            End If
        Next
        myMetadataXml = Nothing
        Return VaultMetadata
    End Function
#End Region


End Class
