Public Module Normalization


    Public Function NormalizeDate(ByVal StrDate As String) As String
        If IsValidDate(StrDate) Then
            Return StrDate
        ElseIf IsReversedDate(StrDate) Then
            Return DateToStr(StrToDate(StrDate))
        ElseIf IsIncompleteDate(StrDate) Then
            Dim Components As String() = StrDate.Split(CChar("-"))
            Try
                Dim Year As String = Components(0).PadLeft(4, CChar("0"))
                Dim Month As String = Components(1).PadLeft(2, CChar("0"))
                Dim Day As String = Components(2).PadLeft(2, CChar("0"))
                Return String.Format("{0}-{1}-{2}", Year, Month, Day)
            Catch ex As Exception
                Return "0000-00-00"
            End Try
        ElseIf IsJustYearDate(StrDate) Then
            Return String.Format("{0}-00-00", StrDate)
        Else
            Return "0000-00-00"
        End If
    End Function




    Public Function GetCompleteDate(ByVal d As String, ByVal DateType As ExtremeDateTypes) As String
        If d Is Nothing OrElse d = "" Then Return "0000-00-00"

        If DateType = ExtremeDateTypes.Initial Then
            If Matches(d, YearMonthDay) Then
                Return d
            ElseIf Matches(d, YearMonth) Then
                Return d & "01"
            ElseIf Matches(d, YearDay) Then
                Return d.Substring(0, 5) & "01" & d.Substring(5, 3)
            ElseIf Matches(d, Year) Then
                Return d.Substring(0, 5) & "01-01"
            End If
        Else ' Final date
            If Matches(d, YearMonthDay) Then
                Return d
            ElseIf Matches(d, YearMonth) Then
                Dim month As String = d.Substring(5, 2)
                Dim day As String
                Select Case month
                    Case "01", "03", "05", "07", "08", "10", "12" : day = "31"
                    Case "02" : day = "28"
                    Case Else : day = "30"
                End Select
                Return d & day
            ElseIf Matches(d, YearDay) Then
                Return d.Substring(0, 5) & "12" & d.Substring(5, 3)
            ElseIf Matches(d, Year) Then
                Return d.Substring(0, 5) & "12-31"
            End If
        End If
        Return d
    End Function


    Public Function FixUnkownDate(ByVal d As String, ByVal DateType As ExtremeDateTypes) As String
        Dim components As String() = d.Split(CChar("-"))
        Dim year As String = components(0)
        Dim month As String = components(1)
        Dim day As String = components(2)

        If DateType = ExtremeDateTypes.Initial Then
            If year = "0000" Then year = "0001"
            If month = "00" Then month = "01"
            If day = "00" Then day = "01"
        Else
            If year = "0000" Then year = "9999"
            If month = "00" Then month = "12"
            If day = "00" Then
                Select Case month
                    Case "01", "03", "05", "07", "08", "10", "12" : day = "31"
                    Case "02" : day = "28"
                    Case Else : day = "30"
                End Select
            End If
        End If
        Return String.Format("{0}-{1}-{2}", year, month, day)
    End Function


End Module
