Imports DigitArq.Data.Entities.MDA
Imports DigitArq.Utility

Public Class MetadataViewer
    Inherits System.Windows.Forms.UserControl

    Private LastLocationY As Integer = 0
    Private ISADAreas As New Collection


#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'UserControl1 overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        '
        'MetadataViewer
        '
        Me.AutoScroll = True
        Me.BackColor = System.Drawing.SystemColors.Control
        Me.Name = "MetadataViewer"
        Me.Size = New System.Drawing.Size(200, 256)

    End Sub

#End Region





    Private Sub MetadataViewer_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        Me.SuspendLayout()
        LoadControls()
        Me.ResumeLayout()
    End Sub

    Private Sub ClearControls()
        ResetScroll()
        LastLocationY = 0
        ISADAreas = New Collection
        Me.Controls.Clear()
    End Sub

    Private Sub ResetScroll()
        Me.AutoScrollPosition = New Point(0, 0)
    End Sub



    Public Sub LoadLazyNode(ByVal Node As LazyNode)
        SuspendLayout()
        ClearControls()
        Add(BuildIdentityArea(Node))
        Add(BuildContextArea(Node))
        Add(BuildContentStructureArea(Node))
        Add(BuildUseArea(Node))
        Add(BuildAlliedMaterialArea(Node))
        Add(BuildNoteArea(Node))
        Add(BuildDescriptionControlArea(Node))
        Me.LoadControls()
        ResumeLayout()
    End Sub



    Private Sub LoadControls()
        Me.Controls.Clear()
        LastLocationY = 0
        Debug.WriteLine("Loading " & ISADAreas.Count & " ISAD Areas into controls")
        For Each Area As ISADArea In ISADAreas
            'Area.SuspendLayout()
            Area.Location = New Point(0, LastLocationY)
            Area.Width = Me.Width - 20
            Me.Controls.Add(Area)
            Debug.WriteLine("Added ISAD area """ & Area.HeaderText & """ with " & Area.Controls.Count & " internal controls")

            Area.Anchor = AnchorStyles.Left Or AnchorStyles.Top Or AnchorStyles.Right
            LastLocationY += Area.Height
            'Area.ResumeLayout()
        Next
    End Sub


    Private Sub Add(ByVal Area As ISADArea)
        If Not Me.Controls.Contains(Area) AndAlso Area.Controls.Count > 1 Then
            ISADAreas.Add(Area)
        End If
    End Sub


    Private Function BuildIdentityArea(ByVal Node As LazyNode) As ISADArea
        Dim ISADArea As New ISADArea(VisualFieldLabel("ISAD.Identification"))

        Dim CompleteUnitID As New PropertyValueViewer
        CompleteUnitID.PropertyLabel = VisualFieldLabel("CompleteUnitId")
        CompleteUnitID.PropertyValue = Node.CompleteUnitId
        ISADArea.Add(CompleteUnitID)


        Dim OtherLevel As New PropertyValueViewer
        OtherLevel.PropertyLabel = VisualFieldLabel("OtherLevel")
        OtherLevel.PropertyValue = VisualFieldLabel(Node.OtherLevel)
        ISADArea.Add(OtherLevel)


        Dim UnitTitle As New PropertyValueViewer
        UnitTitle.PropertyLabel = VisualFieldLabel("UnitTitle")
        UnitTitle.PropertyValue = Node.UnitTitle
        ISADArea.Add(UnitTitle)


        Dim UnitDates As New PropertyValueViewer
        UnitDates.PropertyLabel = VisualFieldLabel("UnitDates")
        UnitDates.PropertyValue = Node.UnitDateInitial & "/" & Node.UnitDateFinal
        ISADArea.Add(UnitDates)

        Dim UnitDateBulk As New PropertyValueViewer
        UnitDates.PropertyLabel = VisualFieldLabel("UnitDateBulk")
        UnitDates.PropertyValue = Node.UnitDateBulk
        ISADArea.Add(UnitDateBulk)

        Dim UnitDateNotes As New PropertyValueViewer
        UnitDates.PropertyLabel = VisualFieldLabel("UnitDateNotes")
        UnitDates.PropertyValue = Node.UnitDateNotes
        ISADArea.Add(UnitDateNotes)


        Dim Dimensions As New PropertyValueViewer
        Dimensions.PropertyLabel = VisualFieldLabel("Dimensions")
        Dimensions.PropertyValue = Node.Dimensions
        ISADArea.Add(Dimensions)

        Dim Repository As New PropertyValueViewer
        Repository.PropertyLabel = VisualFieldLabel("Repository")
        Repository.PropertyValue = Node.Repository
        ISADArea.Add(Repository)


        Dim GenreForm As New PropertyValueViewer
        Repository.PropertyLabel = VisualFieldLabel("GenreForm")
        Repository.PropertyValue = Node.GenreForm
        ISADArea.Add(GenreForm)

        Dim Abstract As New PropertyValueViewer
        Repository.PropertyLabel = VisualFieldLabel("Abstract")
        Repository.PropertyValue = Node.Abstract
        ISADArea.Add(Abstract)

        ISADArea.LoadControls()

        Return ISADArea
    End Function


    Private Function BuildContextArea(ByVal Node As LazyNode) As ISADArea
        Dim ISADArea As New ISADArea(VisualFieldLabel("ISAD.Context"))

        Dim OriginationAuthor As New PropertyValueViewer
        OriginationAuthor.PropertyLabel = VisualFieldLabel("OriginationAuthor")
        OriginationAuthor.PropertyValue = Node.OriginationAuthor
        ISADArea.Add(OriginationAuthor)

        Dim OriginationAuthorAddress As New PropertyValueViewer
        OriginationAuthor.PropertyLabel = VisualFieldLabel("OriginationAuthorAddress")
        OriginationAuthor.PropertyValue = Node.OriginationAuthorAddress
        ISADArea.Add(OriginationAuthorAddress)


        Dim OriginationNotary As New PropertyValueViewer
        OriginationNotary.PropertyLabel = VisualFieldLabel("OriginationNotary")
        OriginationNotary.PropertyValue = Node.OriginationNotary
        ISADArea.Add(OriginationNotary)

        Dim OriginationScrivener As New PropertyValueViewer
        OriginationScrivener.PropertyLabel = VisualFieldLabel("OriginationScrivener")
        OriginationScrivener.PropertyValue = Node.OriginationScrivener
        ISADArea.Add(OriginationScrivener)

        Dim OriginationDestination As New PropertyValueViewer
        OriginationDestination.PropertyLabel = VisualFieldLabel("OriginationDestination")
        OriginationDestination.PropertyValue = Node.OriginationDestination
        ISADArea.Add(OriginationDestination)

        Dim OriginationDestinationAddress As New PropertyValueViewer
        OriginationDestination.PropertyLabel = VisualFieldLabel("OriginationDestinationAddress")
        OriginationDestination.PropertyValue = Node.OriginationDestinationAddress
        ISADArea.Add(OriginationDestinationAddress)

        Dim BiogHist As New PropertyValueViewer
        BiogHist.PropertyLabel = VisualFieldLabel("BiogHist")
        BiogHist.PropertyValue = Node.BiogHist
        ISADArea.Add(BiogHist)

        Dim CustodHist As New PropertyValueViewer
        CustodHist.PropertyLabel = VisualFieldLabel("CustodHist")
        CustodHist.PropertyValue = Node.CustodHist
        ISADArea.Add(CustodHist)

        Dim AcqInfo As New PropertyValueViewer
        AcqInfo.PropertyLabel = VisualFieldLabel("AcqInfo")
        AcqInfo.PropertyValue = Node.AcqInfo
        ISADArea.Add(AcqInfo)

        Dim GeogName As New PropertyValueViewer
        GeogName.PropertyLabel = VisualFieldLabel("GeogName")
        GeogName.PropertyValue = Node.GeogName
        ISADArea.Add(GeogName)


        ISADArea.LoadControls()

        Return ISADArea
    End Function


    Private Function BuildContentStructureArea(ByVal Node As LazyNode) As ISADArea
        Dim ISADArea As New ISADArea(VisualFieldLabel("ISAD.ContentStructure"))

        Dim ScopeContent As New PropertyValueViewer
        ScopeContent.PropertyLabel = VisualFieldLabel("ScopeContent")
        ScopeContent.PropertyValue = Node.ScopeContent
        ISADArea.Add(ScopeContent)

        Dim Appraisal As New PropertyValueViewer
        Appraisal.PropertyLabel = VisualFieldLabel("Appraisal")
        Appraisal.PropertyValue = Node.Appraisal
        ISADArea.Add(Appraisal)


        Dim Accruals As New PropertyValueViewer
        Accruals.PropertyLabel = VisualFieldLabel("Accruals")
        Accruals.PropertyValue = Node.Accruals
        ISADArea.Add(Accruals)

        Dim Arrangement As New PropertyValueViewer
        Arrangement.PropertyLabel = VisualFieldLabel("Arrangement")
        Arrangement.PropertyValue = Node.Arrangement
        ISADArea.Add(Arrangement)

        Dim FilePlan As New PropertyValueViewer
        FilePlan.PropertyLabel = VisualFieldLabel("FilePlan")
        FilePlan.PropertyValue = Node.FilePlan
        ISADArea.Add(FilePlan)

        ISADArea.LoadControls()

        Return ISADArea
    End Function


    Private Function BuildUseArea(ByVal Node As LazyNode) As ISADArea
        Dim ISADArea As New ISADArea(VisualFieldLabel("ISAD.Use"))

        Dim LegalStatus As New PropertyValueViewer
        LegalStatus.PropertyLabel = VisualFieldLabel("LegalStatus")
        LegalStatus.PropertyValue = Node.LegalStatus
        ISADArea.Add(LegalStatus)

        Dim AccessRestrict As New PropertyValueViewer
        AccessRestrict.PropertyLabel = VisualFieldLabel("AccessRestrict")
        AccessRestrict.PropertyValue = Node.AccessRestrict
        ISADArea.Add(AccessRestrict)

        Dim PhysFacet As New PropertyValueViewer
        PhysFacet.PropertyLabel = VisualFieldLabel("PhysFacet")
        PhysFacet.PropertyValue = Node.PhysFacet
        ISADArea.Add(PhysFacet)


        Dim UseRestrict As New PropertyValueViewer
        PhysFacet.PropertyLabel = VisualFieldLabel("UseRestrict")
        PhysFacet.PropertyValue = Node.UseRestrict
        ISADArea.Add(UseRestrict)


        Dim LangMaterial As New PropertyValueViewer
        LangMaterial.PropertyLabel = VisualFieldLabel("LangMaterial")
        LangMaterial.PropertyValue = Node.LangMaterial
        ISADArea.Add(LangMaterial)

        Dim PhysTech As New PropertyValueViewer
        PhysTech.PropertyLabel = VisualFieldLabel("PhysTech")
        PhysTech.PropertyValue = Node.PhysTech
        ISADArea.Add(PhysTech)


        Dim MaterialSpec As New PropertyValueViewer
        PhysTech.PropertyLabel = VisualFieldLabel("MaterialSpec")
        PhysTech.PropertyValue = Node.MaterialSpec
        ISADArea.Add(MaterialSpec)

        Dim OtherFindAid As New PropertyValueViewer
        OtherFindAid.PropertyLabel = VisualFieldLabel("OtherFindAid")
        OtherFindAid.PropertyValue = Node.OtherFindAid
        ISADArea.Add(OtherFindAid)


        Dim PhysLoc As New PropertyValueViewer
        OtherFindAid.PropertyLabel = VisualFieldLabel("PhysLoc")
        OtherFindAid.PropertyValue = Node.PhysLoc
        ISADArea.Add(PhysLoc)


        Dim OriginalNumbering As New PropertyValueViewer
        OtherFindAid.PropertyLabel = VisualFieldLabel("OriginalNumbering")
        OtherFindAid.PropertyValue = Node.OriginalNumbering
        ISADArea.Add(OriginalNumbering)

        Dim PreviousLoc As New PropertyValueViewer
        OtherFindAid.PropertyLabel = VisualFieldLabel("PreviousLoc")
        OtherFindAid.PropertyValue = Node.PreviousLoc
        ISADArea.Add(PreviousLoc)

        ISADArea.LoadControls()

        Return ISADArea
    End Function



    Private Function BuildAlliedMaterialArea(ByVal Node As LazyNode) As ISADArea
        Dim ISADArea As New ISADArea(VisualFieldLabel("ISAD.AlliedMaterial"))

        Dim OriginalsLoc As New PropertyValueViewer
        OriginalsLoc.PropertyLabel = VisualFieldLabel("OriginalsLoc")
        OriginalsLoc.PropertyValue = Node.OriginalsLoc
        ISADArea.Add(OriginalsLoc)

        Dim AltFormAvail As New PropertyValueViewer
        AltFormAvail.PropertyLabel = VisualFieldLabel("AltFormAvail")
        AltFormAvail.PropertyValue = Node.AltFormAvail
        ISADArea.Add(AltFormAvail)

        Dim RelatedMaterial As New PropertyValueViewer
        RelatedMaterial.PropertyLabel = VisualFieldLabel("RelatedMaterial")
        RelatedMaterial.PropertyValue = Node.RelatedMaterial
        ISADArea.Add(RelatedMaterial)

        Dim SeparatedMaterial As New PropertyValueViewer
        SeparatedMaterial.PropertyLabel = VisualFieldLabel("SeparatedMaterial")
        SeparatedMaterial.PropertyValue = Node.SeparatedMaterial
        ISADArea.Add(SeparatedMaterial)

        If Not Node.Bibliography Is Nothing Then
            ' handle multiple bibrefs
            Dim BibliographyString As String = ""
            For Each BibRefItem As BibRefItem In Node.Bibliography
                BibliographyString &= "- " & BibRefItem.BibRef() & vbNewLine
            Next

            Dim Bibliography As New PropertyValueViewer
            Bibliography.PropertyLabel = VisualFieldLabel("Bibliography")
            Bibliography.PropertyValue = BibliographyString
            ISADArea.Add(Bibliography)
        End If

        ISADArea.LoadControls()

        Return ISADArea
    End Function


    Private Function BuildNoteArea(ByVal Node As LazyNode) As ISADArea
        Dim ISADArea As New ISADArea(VisualFieldLabel("ISAD.Note"))

        Dim Note As New PropertyValueViewer
        Note.PropertyLabel = VisualFieldLabel("Note")
        Note.PropertyValue = Node.Note
        ISADArea.Add(Note)


        If Not Node.ArchivistNotes Is Nothing Then
            ' handle multiple archivist notes
            Dim ArchNotesString As String = ""
            For Each ArchNoteItem As ArchivistNotesItem In Node.ArchivistNotes
                ArchNotesString &= "- " & ArchNoteItem.NoteTitle & vbNewLine
            Next


            Dim ArchivistNotes As New PropertyValueViewer
            ArchivistNotes.PropertyLabel = VisualFieldLabel("ArchivistNotes")
            ArchivistNotes.PropertyValue = ArchNotesString
            ISADArea.Add(ArchivistNotes)
        End If

        ISADArea.LoadControls()

        Return ISADArea
    End Function



    Private Function BuildDescriptionControlArea(ByVal Node As LazyNode) As ISADArea
        Dim ISADArea As New ISADArea(VisualFieldLabel("ISAD.DescriptionControl"))

        Dim ProcessInfo As New PropertyValueViewer
        ProcessInfo.PropertyLabel = VisualFieldLabel("ProcessInfo")
        ProcessInfo.PropertyValue = Node.ProcessInfo
        ISADArea.Add(ProcessInfo)


        Dim ProcessInfoName As New PropertyValueViewer
        ProcessInfoName.PropertyLabel = VisualFieldLabel("ProcessInfoName")
        ProcessInfoName.PropertyValue = Node.ProcessInfoName
        ISADArea.Add(ProcessInfoName)


        Dim ProcessInfoDate As New PropertyValueViewer
        ProcessInfoDate.PropertyLabel = VisualFieldLabel("ProcessInfoDate")
        ProcessInfoDate.PropertyValue = Node.ProcessInfoDate
        ISADArea.Add(ProcessInfoDate)


        Dim OtherDescriptiveData As New PropertyValueViewer
        OtherDescriptiveData.PropertyLabel = VisualFieldLabel("OtherDescriptiveData")
        OtherDescriptiveData.PropertyValue = Node.OtherDescriptiveData
        ISADArea.Add(OtherDescriptiveData)

        ISADArea.LoadControls()

        Return ISADArea
    End Function



End Class
